#!/usr/bin/env python

'''
This script converts between Thrift protocol embeddings.

Usage is:
python convert_protocols.py \
  --input-file /path/to/input/file \
  --output-file /path/to/output/file \
  --direction <known conversion>

<known conversion> is a categorical directive, in the form
  <input-encoding>-to-<output-encoding>
The --input-file must be in the input format; the output file will
be in the <output-encoding>

<known conversion> must currently be either
  * binary-to-compact: Convert the TBinaryProtocol encoded input file to
                       a TCompactProtocol encoded output file
  * compact-to-binary: Convert the TCompactProtocol encoded input file
                       to a TBinaryProtocol encoded output file

The known conversions are stored in a dictionary KNOWN_CONVERSIONS:
  * keys are the conversion name, e.g., 'binary-to-compact'
  * values are pairs of callable factory methods
    (callable-input-factory, callable-output-factory), e.g.,
    (TBinaryProtocol.TBinaryProtocolAcceleratedFactory,
     TCompactProtocol.TCompactProtocolAcceleratedFactory)

Therefore, adding to the conversions is simple: just add to the
KNOWN_CONVERSIONS mapping.
'''
from __future__ import print_function
from __future__ import unicode_literals

import argparse
import gzip
import logging
import mimetypes

from thrift import TSerialization
from thrift.protocol import (
    TCompactProtocol, TBinaryProtocol, TJSONProtocol
)
from thrift.transport import TTransport

from concrete import Communication
import concrete.version
from concrete.util import set_stdout_encoding

PROTOCOLS = {
    "binary": TBinaryProtocol.TBinaryProtocolAcceleratedFactory,
    "compact": TCompactProtocol.TCompactProtocolAcceleratedFactory,
    "json": TJSONProtocol.TJSONProtocolFactory
}

KNOWN_CONVERSIONS = {
    'binary-to-compact': (PROTOCOLS["binary"], PROTOCOLS["compact"]),
    'compact-to-binary': (PROTOCOLS["compact"], PROTOCOLS["binary"]),
    'compact-to-json': (PROTOCOLS["compact"], PROTOCOLS["json"]),
    'json-to-compact': (PROTOCOLS["json"], PROTOCOLS["compact"])
}


def make_parser():
    parser = argparse.ArgumentParser(
        description='Convert communications between protocols')
    parser.add_argument('--input-file', type=str, required=True,
                        help='input file path')
    parser.add_argument('--output-file', type=str, required=True,
                        help='output file path')
    parser.add_argument('-l', '--loglevel', '--log-level',
                        help='Logging verbosity level threshold (to stderr)',
                        default='info')
    parser.add_argument(
        '--direction', choices=KNOWN_CONVERSIONS.keys(), required=False)
    parser.add_argument(
        '--iprotocol', choices=sorted(PROTOCOLS.keys()), required=False)
    parser.add_argument(
        '--oprotocol', choices=sorted(PROTOCOLS.keys()), required=False)
    concrete.version.add_argparse_argument(parser)
    return parser


def convert(input_file_path, output_file_path, input_protocol_factory,
            output_protocol_factory):
    """
    Convert an input file (to be read in as an input_protocol_factory
    type) to an output file (with encoding output_protocol_factory
    type).

    * input_file_path: Path to the input file (on disk)
    * output_file_path: Path the output file (on disk)
    * input_protocol_factory: Callable factory function for input
      encoding, e.g., TBinaryProtocol.TBinaryProtocolAcceleratedFactory.
    * output_protocol_factory: Callable factory function for output
      encoding, e.g., TCompactProtocol.TCompactProtocolAcceleratedFactory.
    """
    input_file = open(input_file_path, 'r')
    input_bytes = input_file.read()
    output_bytes = convert_communication(
        input_bytes, input_protocol_factory, output_protocol_factory)
    input_file.close()
    output_file = open(output_file_path, 'w')
    output_file.write(output_bytes)
    output_file.close()


def convert_communication(input_bytes, input_protocol_factory,
                          output_protocol_factory):
    """
    Convert an input byte stream (to be read in as an
    input_protocol_factory type) to an output byte stream (with encoding
    output_protocol_factory type).

    * input_bytes: Input file byte stream
    * input_protocol_factory: Callable factory function for input
      encoding, e.g., TBinaryProtocol.TBinaryProtocolAcceleratedFactory.
    * output_protocol_factory: Callable factory function for output
      encoding, e.g., TCompactProtocol.TCompactProtocolAcceleratedFactory.
    """
    comm = Communication()
    TSerialization.deserialize(comm,
                               input_bytes,
                               protocol_factory=input_protocol_factory())
    output_bytes = TSerialization.serialize(
        comm, protocol_factory=output_protocol_factory())
    return output_bytes


def main():
    set_stdout_encoding()

    parser = make_parser()
    args = parser.parse_args()

    logging.basicConfig(format='%(asctime)-15s %(levelname)s: %(message)s',
                        level=args.loglevel.upper())

    mimetypes.init()
    (ifile_type, ifile_encoding) = mimetypes.guess_type(args.input_file)
    (ofile_type, ofile_encoding) = mimetypes.guess_type(args.output_file)
    out_writer = None
    if args.direction is None:
        if args.iprotocol is None or args.oprotocol is None:
            print("Either --direction, or both --iprotocol and --oprotocol,"
                  " must be provided")
            exit(1)
    else:
        if (args.iprotocol is not None) or (args.oprotocol is not None):
            print("Not both --direction, and either --iprotocol or"
                  " --oprotocol, can be provided")
            exit(1)
    encoding_input = KNOWN_CONVERSIONS[args.direction][
        0] if args.iprotocol is None else PROTOCOLS[args.iprotocol]
    encoding_output = KNOWN_CONVERSIONS[args.direction][
        1] if args.oprotocol is None else PROTOCOLS[args.oprotocol]
    if ofile_encoding == "gzip":
        out_writer = gzip.GzipFile(args.output_file, 'wb')
    else:
        out_writer = open(args.output_file, 'w')
    if ifile_encoding == 'gzip':
        f = gzip.GzipFile(args.input_file)
        transportIn = TTransport.TFileObjectTransport(f)
        protocolIn = encoding_input().getProtocol(transportIn)
        while True:
            try:
                comm = Communication()
                comm.read(protocolIn)
                output_bytes = TSerialization.serialize(
                    comm, protocol_factory=encoding_output())
                out_writer.write(output_bytes)
            except EOFError:
                break
        f.close()
    else:
        convert(input_file_path=args.input_file,
                output_file_path=args.output_file,
                input_protocol_factory=encoding_input,
                output_protocol_factory=encoding_output)
    out_writer.close()


if __name__ == '__main__':
    main()



#!/usr/bin/env python

from __future__ import print_function
from __future__ import unicode_literals

import argparse
import logging

from boto import connect_s3
import concrete.version
from concrete.util import (
    set_stdout_encoding, S3BackedCommunicationContainer,
    DEFAULT_S3_KEY_PREFIX_LEN,
)


def main():
    set_stdout_encoding()

    parser = argparse.ArgumentParser(
        description='Read communications from AWS S3 bucket (keyed by '
                    'communication id) and write to a tar.gz file.',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    parser.add_argument('bucket_name', help='name of S3 bucket to read from')
    parser.add_argument('--prefix-len', type=int, default=DEFAULT_S3_KEY_PREFIX_LEN,
                        help='S3 keys are prefixed with hashes of this length')
    parser.add_argument('-l', '--loglevel', '--log-level',
                        help='Logging verbosity level threshold (to stderr)',
                        default='info')
    concrete.version.add_argparse_argument(parser)
    args = parser.parse_args()

    logging.basicConfig(format='%(asctime)-15s %(levelname)s: %(message)s',
                        level=args.loglevel.upper())

    logging.info('connecting to s3')
    conn = connect_s3()
    logging.info('retrieving bucket {}'.format(args.bucket_name))
    bucket = conn.get_bucket(args.bucket_name)
    logging.info('reading from s3 bucket {}, prefix length {}'.format(
        args.bucket_name, args.prefix_len))
    container = S3BackedCommunicationContainer(bucket, args.prefix_len)
    for comm_id in container:
        print(comm_id)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
#
# Autogenerated by Thrift Compiler (0.11.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:coding=utf-8
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec

import sys
import concrete.uuid.ttypes
import concrete.metadata.ttypes

from thrift.transport import TTransport
all_structs = []


class LinkTarget(object):
    """
    A structure that represents the target of an entity linking annotation.

    Attributes:
     - confidence: Confidence of this LinkTarget object.
     - targetId: A UUID that represents the target of this LinkTarget. This
    UUID should exist in the Entity/Situation(Mention)Set that the
    Linking object is contained in.
     - dbId: A database ID that represents the target of this linking.

    This should be used if the target of the linking is not associated
    with an Entity|Situation(Mention)Set in Concrete, and therefore cannot be linked by
    a UUID internal to concrete.

    If present, other optional field 'dbName' should also be populated.
     - dbName: The name of the database that represents the target of this linking.

    Together with the 'dbId', this can form a pointer to a target
    that is not represented inside concrete.

    Should be populated alongside 'dbId'.
    """


    def __init__(self, confidence=None, targetId=None, dbId=None, dbName=None,):
        self.confidence = confidence
        self.targetId = targetId
        self.dbId = dbId
        self.dbName = dbName

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.DOUBLE:
                    self.confidence = iprot.readDouble()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRUCT:
                    self.targetId = concrete.uuid.ttypes.UUID()
                    self.targetId.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.STRING:
                    self.dbId = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.STRING:
                    self.dbName = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('LinkTarget')
        if self.confidence is not None:
            oprot.writeFieldBegin('confidence', TType.DOUBLE, 1)
            oprot.writeDouble(self.confidence)
            oprot.writeFieldEnd()
        if self.targetId is not None:
            oprot.writeFieldBegin('targetId', TType.STRUCT, 2)
            self.targetId.write(oprot)
            oprot.writeFieldEnd()
        if self.dbId is not None:
            oprot.writeFieldBegin('dbId', TType.STRING, 3)
            oprot.writeString(self.dbId.encode('utf-8') if sys.version_info[0] == 2 else self.dbId)
            oprot.writeFieldEnd()
        if self.dbName is not None:
            oprot.writeFieldBegin('dbName', TType.STRING, 4)
            oprot.writeString(self.dbName.encode('utf-8') if sys.version_info[0] == 2 else self.dbName)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class Link(object):
    """
    A structure that represents the origin of an entity linking annotation.

    Attributes:
     - sourceId: The "root" of this Link; points to a EntityMention UUID, Entity UUID, etc.
     - linkTargetList: A list of LinkTarget objects that this Link contains.
    """


    def __init__(self, sourceId=None, linkTargetList=None,):
        self.sourceId = sourceId
        self.linkTargetList = linkTargetList

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.sourceId = concrete.uuid.ttypes.UUID()
                    self.sourceId.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.LIST:
                    self.linkTargetList = []
                    (_etype3, _size0) = iprot.readListBegin()
                    for _i4 in range(_size0):
                        _elem5 = LinkTarget()
                        _elem5.read(iprot)
                        self.linkTargetList.append(_elem5)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('Link')
        if self.sourceId is not None:
            oprot.writeFieldBegin('sourceId', TType.STRUCT, 1)
            self.sourceId.write(oprot)
            oprot.writeFieldEnd()
        if self.linkTargetList is not None:
            oprot.writeFieldBegin('linkTargetList', TType.LIST, 2)
            oprot.writeListBegin(TType.STRUCT, len(self.linkTargetList))
            for iter6 in self.linkTargetList:
                iter6.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.sourceId is None:
            raise TProtocolException(message='Required field sourceId is unset!')
        if self.linkTargetList is None:
            raise TProtocolException(message='Required field linkTargetList is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class Linking(object):
    """
    A structure that represents entity linking annotations.

    Attributes:
     - metadata: Metadata related to this Linking object.
     - linkList: A list of Link objects that this Linking object contains.
    """


    def __init__(self, metadata=None, linkList=None,):
        self.metadata = metadata
        self.linkList = linkList

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.metadata = concrete.metadata.ttypes.AnnotationMetadata()
                    self.metadata.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.LIST:
                    self.linkList = []
                    (_etype10, _size7) = iprot.readListBegin()
                    for _i11 in range(_size7):
                        _elem12 = Link()
                        _elem12.read(iprot)
                        self.linkList.append(_elem12)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('Linking')
        if self.metadata is not None:
            oprot.writeFieldBegin('metadata', TType.STRUCT, 1)
            self.metadata.write(oprot)
            oprot.writeFieldEnd()
        if self.linkList is not None:
            oprot.writeFieldBegin('linkList', TType.LIST, 2)
            oprot.writeListBegin(TType.STRUCT, len(self.linkList))
            for iter13 in self.linkList:
                iter13.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.metadata is None:
            raise TProtocolException(message='Required field metadata is unset!')
        if self.linkList is None:
            raise TProtocolException(message='Required field linkList is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(LinkTarget)
LinkTarget.thrift_spec = (
    None,  # 0
    (1, TType.DOUBLE, 'confidence', None, None, ),  # 1
    (2, TType.STRUCT, 'targetId', [concrete.uuid.ttypes.UUID, None], None, ),  # 2
    (3, TType.STRING, 'dbId', 'UTF8', None, ),  # 3
    (4, TType.STRING, 'dbName', 'UTF8', None, ),  # 4
)
all_structs.append(Link)
Link.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'sourceId', [concrete.uuid.ttypes.UUID, None], None, ),  # 1
    (2, TType.LIST, 'linkTargetList', (TType.STRUCT, [LinkTarget, None], False), None, ),  # 2
)
all_structs.append(Linking)
Linking.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'metadata', [concrete.metadata.ttypes.AnnotationMetadata, None], None, ),  # 1
    (2, TType.LIST, 'linkList', (TType.STRUCT, [Link, None], False), None, ),  # 2
)
fix_spec(all_structs)
del all_structs



from __future__ import unicode_literals
from pytest import fixture, mark
from concrete.util import CommunicationReader
from concrete.validate import validate_communication
from concrete.util import compress_uuids
import os
import sys
from subprocess import Popen, PIPE
from tempfile import mkstemp


@fixture
def output_file(request):
    (fd, path) = mkstemp()
    os.close(fd)

    def _remove():
        if os.path.exists(path):
            os.remove(path)

    request.addfinalizer(_remove)
    return path


@mark.parametrize('args', [
    (),
    ('--verify',),
    ('--verify', '--single-analytic'),
    ('--single-analytic',),
])
def test_compress_uuids(output_file, args):
    input_file = 'tests/testdata/simple.tar.gz'

    p = Popen([
        sys.executable,
        'scripts/compress-uuids.py',
        input_file,
        output_file
    ] + list(args), stdout=PIPE, stderr=PIPE)
    (stdout, stderr) = p.communicate()

    assert p.returncode == 0

    reader = CommunicationReader(output_file)
    it = iter(reader)

    (comm, comm_filename) = next(it)
    assert comm_filename == 'simple_1.concrete'
    assert comm.id == 'one'
    assert validate_communication(comm)

    (comm, comm_filename) = next(it)
    assert comm_filename == 'simple_2.concrete'
    assert comm.id == 'two'
    assert validate_communication(comm)

    (comm, comm_filename) = next(it)
    assert comm_filename == 'simple_3.concrete'
    assert comm.id == 'three'
    assert validate_communication(comm)

    assert os.stat(output_file).st_size < os.stat(input_file).st_size

    try:
        next(it)
    except StopIteration:
        pass
    else:
        assert False


@mark.parametrize('reader_kwargs,compress_kwargs', [
    (dict(add_references=False), dict()),
    (dict(add_references=False), dict(verify=True)),
    (dict(add_references=False), dict(verify=False)),
    (dict(add_references=False), dict(verify=True, single_analytic=True)),
    (dict(add_references=False), dict(verify=False, single_analytic=True)),
    (dict(add_references=False), dict(verify=True, single_analytic=False)),
    (dict(add_references=False), dict(verify=False, single_analytic=False)),
    (dict(add_references=False), dict(single_analytic=False)),
    (dict(add_references=False), dict(single_analytic=True)),
    (dict(add_references=True), dict()),
    mark.xfail(
        (dict(add_references=True), dict(verify=True)),
        strict=True,
    ),
    (dict(add_references=True), dict(verify=False)),
    mark.xfail(
        (dict(add_references=True), dict(verify=True, single_analytic=True)),
        strict=True,
    ),
    (dict(add_references=True), dict(verify=False, single_analytic=True)),
    mark.xfail(
        (dict(add_references=True), dict(verify=True, single_analytic=False)),
        strict=True,
    ),
    (dict(add_references=True), dict(verify=False, single_analytic=False)),
    (dict(add_references=True), dict(single_analytic=False)),
    (dict(add_references=True), dict(single_analytic=True)),
])
def test_compress_uuids_api(reader_kwargs, compress_kwargs):
    input_file = 'tests/testdata/simple.tar.gz'
    reader = CommunicationReader(input_file, **reader_kwargs)
    it = iter(reader)

    (comm, _) = next(it)
    (new_comm, uc) = compress_uuids(comm, **compress_kwargs)
    assert new_comm.id == 'one'
    assert comm.id == new_comm.id
    assert validate_communication(new_comm)

    (comm, _) = next(it)
    (new_comm, uc) = compress_uuids(comm, **compress_kwargs)
    assert new_comm.id == 'two'
    assert comm.id == new_comm.id
    assert validate_communication(new_comm)

    (comm, _) = next(it)
    (new_comm, uc) = compress_uuids(comm, **compress_kwargs)
    assert new_comm.id == 'three'
    assert comm.id == new_comm.id
    assert validate_communication(new_comm)

    try:
        next(it)
    except StopIteration:
        pass
    else:
        assert False



from __future__ import unicode_literals
from thrift.transport.TTransport import TMemoryBuffer

from .thrift_factory import factory
from .references import add_references_to_communication
from ..communication.ttypes import Communication


def read_communication_from_buffer(buf, add_references=True):
    '''
    Deserialize buf (a binary string) and return resulting
    communication.  Add references if requested.

    Args:
        buf (str): String representing communication encoded from thrift
        add_references (bool): If True, calls
           :func:`concrete.util.references.add_references_to_communication`
           on :class:`.Communication` read from buffer

    Returns:
        Communication: Communication read from buffer
    '''
    transport_in = TMemoryBuffer(buf)
    protocol_in = factory.createProtocol(transport_in)
    comm = Communication()
    comm.read(protocol_in)
    if add_references:
        add_references_to_communication(comm)
    return comm


def write_communication_to_buffer(comm):
    '''
    Serialize communication to buffer (binary string) and return
    buffer.

    Args:
        comm (Communication): communication to serialize

    Returns:
        Communication: Communication read from buffer
    '''
    transport = TMemoryBuffer()
    protocol = factory.createProtocol(transport)
    comm.write(protocol)
    return transport.getvalue()


def communication_deep_copy(comm):
    '''
    Return deep copy of communication.

    Args:
        comm (Communication): communication to copy

    Returns:
        Communication: deep copy of comm
    '''
    return read_communication_from_buffer(
        write_communication_to_buffer(comm), add_references=False
    )



# -*- coding: utf-8 -*-
#
# Autogenerated by Thrift Compiler (0.11.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:coding=utf-8
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec

import sys
import concrete.communication.ttypes
import concrete.services.ttypes
import concrete.structure.ttypes
import concrete.uuid.ttypes

from thrift.transport import TTransport
all_structs = []


class SummarySourceType(object):
    DOCUMENT = 0
    TOKENIZATION = 1
    ENTITY = 2

    _VALUES_TO_NAMES = {
        0: "DOCUMENT",
        1: "TOKENIZATION",
        2: "ENTITY",
    }

    _NAMES_TO_VALUES = {
        "DOCUMENT": 0,
        "TOKENIZATION": 1,
        "ENTITY": 2,
    }


class SummarizationRequest(object):
    """
    A request to summarize which specifies the length of the desired
    summary and the text data to be summarized.
    Either set sourceCommunication or sourceType and sourceIds.

    Attributes:
     - queryTerms: Terms or features pertinent to the query.
    Can be empty, meaning summarize all source material with
    no a priori beliefs about what is important to summarize.
     - maximumTokens: Limit on how long the returned summary can be in tokens.
     - maximumCharacters: Limit on how long the returned summary can be in characters.
     - sourceType: How to interpret the ids in sourceIds.
    May be null is sourceIds is null, otherwise must be populated.
     - sourceIds: A list of concrete object ids which serve as the material
    to summarize.
     - sourceCommunication: Alternative to sourceIds+sourceType: provide a Communication
    of text to summarize.
    """


    def __init__(self, queryTerms=None, maximumTokens=None, maximumCharacters=None, sourceType=None, sourceIds=None, sourceCommunication=None,):
        self.queryTerms = queryTerms
        self.maximumTokens = maximumTokens
        self.maximumCharacters = maximumCharacters
        self.sourceType = sourceType
        self.sourceIds = sourceIds
        self.sourceCommunication = sourceCommunication

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.LIST:
                    self.queryTerms = []
                    (_etype3, _size0) = iprot.readListBegin()
                    for _i4 in range(_size0):
                        _elem5 = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                        self.queryTerms.append(_elem5)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.I32:
                    self.maximumTokens = iprot.readI32()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.I32:
                    self.maximumCharacters = iprot.readI32()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.I32:
                    self.sourceType = iprot.readI32()
                else:
                    iprot.skip(ftype)
            elif fid == 5:
                if ftype == TType.LIST:
                    self.sourceIds = []
                    (_etype9, _size6) = iprot.readListBegin()
                    for _i10 in range(_size6):
                        _elem11 = concrete.uuid.ttypes.UUID()
                        _elem11.read(iprot)
                        self.sourceIds.append(_elem11)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 6:
                if ftype == TType.STRUCT:
                    self.sourceCommunication = concrete.communication.ttypes.Communication()
                    self.sourceCommunication.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('SummarizationRequest')
        if self.queryTerms is not None:
            oprot.writeFieldBegin('queryTerms', TType.LIST, 1)
            oprot.writeListBegin(TType.STRING, len(self.queryTerms))
            for iter12 in self.queryTerms:
                oprot.writeString(iter12.encode('utf-8') if sys.version_info[0] == 2 else iter12)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.maximumTokens is not None:
            oprot.writeFieldBegin('maximumTokens', TType.I32, 2)
            oprot.writeI32(self.maximumTokens)
            oprot.writeFieldEnd()
        if self.maximumCharacters is not None:
            oprot.writeFieldBegin('maximumCharacters', TType.I32, 3)
            oprot.writeI32(self.maximumCharacters)
            oprot.writeFieldEnd()
        if self.sourceType is not None:
            oprot.writeFieldBegin('sourceType', TType.I32, 4)
            oprot.writeI32(self.sourceType)
            oprot.writeFieldEnd()
        if self.sourceIds is not None:
            oprot.writeFieldBegin('sourceIds', TType.LIST, 5)
            oprot.writeListBegin(TType.STRUCT, len(self.sourceIds))
            for iter13 in self.sourceIds:
                iter13.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.sourceCommunication is not None:
            oprot.writeFieldBegin('sourceCommunication', TType.STRUCT, 6)
            self.sourceCommunication.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class SummaryConcept(object):
    """
    A mention of a concept described in a summary which is thought
    to be informative. Concepts might be named entities, facts, or
    events which were determined to be salient in the text being
    summarized.

    Attributes:
     - tokens: Location in summaryCommunication of this concept
     - concept: Short description of the concept being evoked, e.g. "kbrel:bornIn" or "related:ACME_Corp"
     - confidence: How confident is the system that this concept was evoked by this mention, in [0,1]
     - utility: How informative/important it is that this concept be included in the summary (non-negative).
    """


    def __init__(self, tokens=None, concept=None, confidence=1, utility=1,):
        self.tokens = tokens
        self.concept = concept
        self.confidence = confidence
        self.utility = utility

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.tokens = concrete.structure.ttypes.TokenRefSequence()
                    self.tokens.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRING:
                    self.concept = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.DOUBLE:
                    self.confidence = iprot.readDouble()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.DOUBLE:
                    self.utility = iprot.readDouble()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('SummaryConcept')
        if self.tokens is not None:
            oprot.writeFieldBegin('tokens', TType.STRUCT, 1)
            self.tokens.write(oprot)
            oprot.writeFieldEnd()
        if self.concept is not None:
            oprot.writeFieldBegin('concept', TType.STRING, 2)
            oprot.writeString(self.concept.encode('utf-8') if sys.version_info[0] == 2 else self.concept)
            oprot.writeFieldEnd()
        if self.confidence is not None:
            oprot.writeFieldBegin('confidence', TType.DOUBLE, 3)
            oprot.writeDouble(self.confidence)
            oprot.writeFieldEnd()
        if self.utility is not None:
            oprot.writeFieldBegin('utility', TType.DOUBLE, 4)
            oprot.writeDouble(self.utility)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class Summary(object):
    """
    A shortened version of some text, possibly with some concepts
    annotated as justifications for why particular pieces of the
    summary were kept.

    Attributes:
     - summaryCommunication: Contains the text of the generated summary.
     - concepts: Concepts mentioned in the summary which are believed to be
    interesting and/or worth highlighting.
    """


    def __init__(self, summaryCommunication=None, concepts=None,):
        self.summaryCommunication = summaryCommunication
        self.concepts = concepts

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.summaryCommunication = concrete.communication.ttypes.Communication()
                    self.summaryCommunication.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.LIST:
                    self.concepts = []
                    (_etype17, _size14) = iprot.readListBegin()
                    for _i18 in range(_size14):
                        _elem19 = SummaryConcept()
                        _elem19.read(iprot)
                        self.concepts.append(_elem19)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('Summary')
        if self.summaryCommunication is not None:
            oprot.writeFieldBegin('summaryCommunication', TType.STRUCT, 1)
            self.summaryCommunication.write(oprot)
            oprot.writeFieldEnd()
        if self.concepts is not None:
            oprot.writeFieldBegin('concepts', TType.LIST, 2)
            oprot.writeListBegin(TType.STRUCT, len(self.concepts))
            for iter20 in self.concepts:
                iter20.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class SummarizationCapability(object):
    """
    Attributes:
     - type
     - lang
    """


    def __init__(self, type=None, lang=None,):
        self.type = type
        self.lang = lang

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.I32:
                    self.type = iprot.readI32()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRING:
                    self.lang = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('SummarizationCapability')
        if self.type is not None:
            oprot.writeFieldBegin('type', TType.I32, 1)
            oprot.writeI32(self.type)
            oprot.writeFieldEnd()
        if self.lang is not None:
            oprot.writeFieldBegin('lang', TType.STRING, 2)
            oprot.writeString(self.lang.encode('utf-8') if sys.version_info[0] == 2 else self.lang)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.type is None:
            raise TProtocolException(message='Required field type is unset!')
        if self.lang is None:
            raise TProtocolException(message='Required field lang is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(SummarizationRequest)
SummarizationRequest.thrift_spec = (
    None,  # 0
    (1, TType.LIST, 'queryTerms', (TType.STRING, 'UTF8', False), None, ),  # 1
    (2, TType.I32, 'maximumTokens', None, None, ),  # 2
    (3, TType.I32, 'maximumCharacters', None, None, ),  # 3
    (4, TType.I32, 'sourceType', None, None, ),  # 4
    (5, TType.LIST, 'sourceIds', (TType.STRUCT, [concrete.uuid.ttypes.UUID, None], False), None, ),  # 5
    (6, TType.STRUCT, 'sourceCommunication', [concrete.communication.ttypes.Communication, None], None, ),  # 6
)
all_structs.append(SummaryConcept)
SummaryConcept.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'tokens', [concrete.structure.ttypes.TokenRefSequence, None], None, ),  # 1
    (2, TType.STRING, 'concept', 'UTF8', None, ),  # 2
    (3, TType.DOUBLE, 'confidence', None, 1, ),  # 3
    (4, TType.DOUBLE, 'utility', None, 1, ),  # 4
)
all_structs.append(Summary)
Summary.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'summaryCommunication', [concrete.communication.ttypes.Communication, None], None, ),  # 1
    (2, TType.LIST, 'concepts', (TType.STRUCT, [SummaryConcept, None], False), None, ),  # 2
)
all_structs.append(SummarizationCapability)
SummarizationCapability.thrift_spec = (
    None,  # 0
    (1, TType.I32, 'type', None, None, ),  # 1
    (2, TType.STRING, 'lang', 'UTF8', None, ),  # 2
)
fix_spec(all_structs)
del all_structs



# -*- coding: utf-8 -*-
#
# Autogenerated by Thrift Compiler (0.11.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:coding=utf-8
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec

import sys
import concrete.structure.ttypes
import concrete.metadata.ttypes
import concrete.uuid.ttypes
import concrete.linking.ttypes

from thrift.transport import TTransport
all_structs = []


class Entity(object):
    """
    A single referent (or "entity") that is referred to at least once
    in a given communication, along with pointers to all of the
    references to that referent. The referent's type (e.g., is it a
    person, or a location, or an organization, etc) is also recorded.

    Because each Entity contains pointers to all references to a
    referent with a given communication, an Entity can be
    thought of as a coreference set.

    Attributes:
     - uuid: Unique identifier for this entity.
     - id: A corpus-specific and stable id such as a Freebase mid
    or a DBpedia id.
     - mentionIdList: An list of pointers to all of the mentions of this Entity's
    referent.  (type=EntityMention)
     - rawMentionList: An list of pointers to all of the sentences which contain a
    mention of this Entity.
     - type: The basic type of this entity's referent.
     - confidence: Confidence score for this individual entity.  You can also set a
    confidence score for an entire EntitySet using the EntitySet's
    metadata.
     - canonicalName: A string containing a representative, canonical, or "best" name
    for this entity's referent.  This string may match one of the
    mentions' text strings, but it is not required to.
    """


    def __init__(self, uuid=None, id=None, mentionIdList=None, rawMentionList=None, type=None, confidence=None, canonicalName=None,):
        self.uuid = uuid
        self.id = id
        self.mentionIdList = mentionIdList
        self.rawMentionList = rawMentionList
        self.type = type
        self.confidence = confidence
        self.canonicalName = canonicalName

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.uuid = concrete.uuid.ttypes.UUID()
                    self.uuid.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 6:
                if ftype == TType.STRING:
                    self.id = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.LIST:
                    self.mentionIdList = []
                    (_etype3, _size0) = iprot.readListBegin()
                    for _i4 in range(_size0):
                        _elem5 = concrete.uuid.ttypes.UUID()
                        _elem5.read(iprot)
                        self.mentionIdList.append(_elem5)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 7:
                if ftype == TType.LIST:
                    self.rawMentionList = []
                    (_etype9, _size6) = iprot.readListBegin()
                    for _i10 in range(_size6):
                        _elem11 = concrete.structure.ttypes.TokenRefSequence()
                        _elem11.read(iprot)
                        self.rawMentionList.append(_elem11)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.STRING:
                    self.type = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.DOUBLE:
                    self.confidence = iprot.readDouble()
                else:
                    iprot.skip(ftype)
            elif fid == 5:
                if ftype == TType.STRING:
                    self.canonicalName = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('Entity')
        if self.uuid is not None:
            oprot.writeFieldBegin('uuid', TType.STRUCT, 1)
            self.uuid.write(oprot)
            oprot.writeFieldEnd()
        if self.mentionIdList is not None:
            oprot.writeFieldBegin('mentionIdList', TType.LIST, 2)
            oprot.writeListBegin(TType.STRUCT, len(self.mentionIdList))
            for iter12 in self.mentionIdList:
                iter12.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.type is not None:
            oprot.writeFieldBegin('type', TType.STRING, 3)
            oprot.writeString(self.type.encode('utf-8') if sys.version_info[0] == 2 else self.type)
            oprot.writeFieldEnd()
        if self.confidence is not None:
            oprot.writeFieldBegin('confidence', TType.DOUBLE, 4)
            oprot.writeDouble(self.confidence)
            oprot.writeFieldEnd()
        if self.canonicalName is not None:
            oprot.writeFieldBegin('canonicalName', TType.STRING, 5)
            oprot.writeString(self.canonicalName.encode('utf-8') if sys.version_info[0] == 2 else self.canonicalName)
            oprot.writeFieldEnd()
        if self.id is not None:
            oprot.writeFieldBegin('id', TType.STRING, 6)
            oprot.writeString(self.id.encode('utf-8') if sys.version_info[0] == 2 else self.id)
            oprot.writeFieldEnd()
        if self.rawMentionList is not None:
            oprot.writeFieldBegin('rawMentionList', TType.LIST, 7)
            oprot.writeListBegin(TType.STRUCT, len(self.rawMentionList))
            for iter13 in self.rawMentionList:
                iter13.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.uuid is None:
            raise TProtocolException(message='Required field uuid is unset!')
        if self.mentionIdList is None:
            raise TProtocolException(message='Required field mentionIdList is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class EntitySet(object):
    """
    A theory about the set of entities that are present in a
    message. See also: Entity.

    Attributes:
     - uuid: Unique identifier for this set.
     - metadata: Information about where this set came from.
     - entityList: List of entities in this set.
     - linkingList: Entity linking annotations associated with this EntitySet.
     - mentionSetId: An optional UUID pointer to an EntityMentionSet.

    If this field is present, consumers can assume that all
    Entity objects in this EntitySet have EntityMentions that are included
    in the named EntityMentionSet.
    """


    def __init__(self, uuid=None, metadata=None, entityList=None, linkingList=None, mentionSetId=None,):
        self.uuid = uuid
        self.metadata = metadata
        self.entityList = entityList
        self.linkingList = linkingList
        self.mentionSetId = mentionSetId

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.uuid = concrete.uuid.ttypes.UUID()
                    self.uuid.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRUCT:
                    self.metadata = concrete.metadata.ttypes.AnnotationMetadata()
                    self.metadata.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.LIST:
                    self.entityList = []
                    (_etype17, _size14) = iprot.readListBegin()
                    for _i18 in range(_size14):
                        _elem19 = Entity()
                        _elem19.read(iprot)
                        self.entityList.append(_elem19)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.LIST:
                    self.linkingList = []
                    (_etype23, _size20) = iprot.readListBegin()
                    for _i24 in range(_size20):
                        _elem25 = concrete.linking.ttypes.Linking()
                        _elem25.read(iprot)
                        self.linkingList.append(_elem25)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 5:
                if ftype == TType.STRUCT:
                    self.mentionSetId = concrete.uuid.ttypes.UUID()
                    self.mentionSetId.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('EntitySet')
        if self.uuid is not None:
            oprot.writeFieldBegin('uuid', TType.STRUCT, 1)
            self.uuid.write(oprot)
            oprot.writeFieldEnd()
        if self.metadata is not None:
            oprot.writeFieldBegin('metadata', TType.STRUCT, 2)
            self.metadata.write(oprot)
            oprot.writeFieldEnd()
        if self.entityList is not None:
            oprot.writeFieldBegin('entityList', TType.LIST, 3)
            oprot.writeListBegin(TType.STRUCT, len(self.entityList))
            for iter26 in self.entityList:
                iter26.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.linkingList is not None:
            oprot.writeFieldBegin('linkingList', TType.LIST, 4)
            oprot.writeListBegin(TType.STRUCT, len(self.linkingList))
            for iter27 in self.linkingList:
                iter27.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.mentionSetId is not None:
            oprot.writeFieldBegin('mentionSetId', TType.STRUCT, 5)
            self.mentionSetId.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.uuid is None:
            raise TProtocolException(message='Required field uuid is unset!')
        if self.metadata is None:
            raise TProtocolException(message='Required field metadata is unset!')
        if self.entityList is None:
            raise TProtocolException(message='Required field entityList is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class EntityMention(object):
    """
    A span of text with a specific referent, such as a person,
    organization, or time. Things that can be referred to by a mention
    are called "entities."

    It is left up to individual EntityMention taggers to decide which
    referent types and phrase types to identify. For example, some
    EntityMention taggers may only identify proper nouns, or may only
    identify EntityMentions that refer to people.

    Each EntityMention consists of a sequence of tokens. This sequence
    is usually annotated with information about the referent type
    (e.g., is it a person, or a location, or an organization, etc) as
    well as the phrase type (is it a name, pronoun, common noun, etc.).

    EntityMentions typically consist of a single noun phrase; however,
    other phrase types may also be marked as mentions. For
    example, in the phrase "French hotel," the adjective "French" might
    be marked as a mention for France.

    Attributes:
     - uuid
     - tokens: Pointer to sequence of tokens.

    Special note: In the case of PRO-drop, where there is no explicit
    mention, but an EntityMention is needed for downstream Entity
    analysis, this field should be set to a TokenRefSequence with an
    empty tokenIndexList and the anchorTokenIndex set to the head/only
    token of the verb/predicate from which the PRO was dropped.
     - entityType: The type of referent that is referred to by this mention.
     - phraseType: The phrase type of the tokens that constitute this mention.
     - confidence: A confidence score for this individual mention.  You can also
    set a confidence score for an entire EntityMentionSet using the
    EntityMentionSet's metadata.
     - text: The text content of this entity mention.  This field is
    typically redundant with the string formed by cross-referencing
    the 'tokens.tokenIndexList' field with this mention's
    tokenization. This field may not be generated by all analytics.
     - childMentionIdList: A list of pointers to the "child" EntityMentions of this
    EntityMention.
    """


    def __init__(self, uuid=None, tokens=None, entityType=None, phraseType=None, confidence=None, text=None, childMentionIdList=None,):
        self.uuid = uuid
        self.tokens = tokens
        self.entityType = entityType
        self.phraseType = phraseType
        self.confidence = confidence
        self.text = text
        self.childMentionIdList = childMentionIdList

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.uuid = concrete.uuid.ttypes.UUID()
                    self.uuid.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRUCT:
                    self.tokens = concrete.structure.ttypes.TokenRefSequence()
                    self.tokens.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.STRING:
                    self.entityType = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.STRING:
                    self.phraseType = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 5:
                if ftype == TType.DOUBLE:
                    self.confidence = iprot.readDouble()
                else:
                    iprot.skip(ftype)
            elif fid == 6:
                if ftype == TType.STRING:
                    self.text = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 7:
                if ftype == TType.LIST:
                    self.childMentionIdList = []
                    (_etype31, _size28) = iprot.readListBegin()
                    for _i32 in range(_size28):
                        _elem33 = concrete.uuid.ttypes.UUID()
                        _elem33.read(iprot)
                        self.childMentionIdList.append(_elem33)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('EntityMention')
        if self.uuid is not None:
            oprot.writeFieldBegin('uuid', TType.STRUCT, 1)
            self.uuid.write(oprot)
            oprot.writeFieldEnd()
        if self.tokens is not None:
            oprot.writeFieldBegin('tokens', TType.STRUCT, 2)
            self.tokens.write(oprot)
            oprot.writeFieldEnd()
        if self.entityType is not None:
            oprot.writeFieldBegin('entityType', TType.STRING, 3)
            oprot.writeString(self.entityType.encode('utf-8') if sys.version_info[0] == 2 else self.entityType)
            oprot.writeFieldEnd()
        if self.phraseType is not None:
            oprot.writeFieldBegin('phraseType', TType.STRING, 4)
            oprot.writeString(self.phraseType.encode('utf-8') if sys.version_info[0] == 2 else self.phraseType)
            oprot.writeFieldEnd()
        if self.confidence is not None:
            oprot.writeFieldBegin('confidence', TType.DOUBLE, 5)
            oprot.writeDouble(self.confidence)
            oprot.writeFieldEnd()
        if self.text is not None:
            oprot.writeFieldBegin('text', TType.STRING, 6)
            oprot.writeString(self.text.encode('utf-8') if sys.version_info[0] == 2 else self.text)
            oprot.writeFieldEnd()
        if self.childMentionIdList is not None:
            oprot.writeFieldBegin('childMentionIdList', TType.LIST, 7)
            oprot.writeListBegin(TType.STRUCT, len(self.childMentionIdList))
            for iter34 in self.childMentionIdList:
                iter34.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.uuid is None:
            raise TProtocolException(message='Required field uuid is unset!')
        if self.tokens is None:
            raise TProtocolException(message='Required field tokens is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class EntityMentionSet(object):
    """
    A theory about the set of entity mentions that are present in a
    message. See also: EntityMention

    This type does not represent a coreference relationship, which is handled by Entity.
    This type is meant to represent the output of a entity-mention-identifier,
    which is often a part of an in-doc coreference system.

    Attributes:
     - uuid: Unique identifier for this set.
     - metadata: Information about where this set came from.
     - mentionList: List of mentions in this set.
     - linkingList: Entity linking annotations associated with this EntityMentionSet.
    """


    def __init__(self, uuid=None, metadata=None, mentionList=None, linkingList=None,):
        self.uuid = uuid
        self.metadata = metadata
        self.mentionList = mentionList
        self.linkingList = linkingList

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.uuid = concrete.uuid.ttypes.UUID()
                    self.uuid.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRUCT:
                    self.metadata = concrete.metadata.ttypes.AnnotationMetadata()
                    self.metadata.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.LIST:
                    self.mentionList = []
                    (_etype38, _size35) = iprot.readListBegin()
                    for _i39 in range(_size35):
                        _elem40 = EntityMention()
                        _elem40.read(iprot)
                        self.mentionList.append(_elem40)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.LIST:
                    self.linkingList = []
                    (_etype44, _size41) = iprot.readListBegin()
                    for _i45 in range(_size41):
                        _elem46 = concrete.linking.ttypes.Linking()
                        _elem46.read(iprot)
                        self.linkingList.append(_elem46)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('EntityMentionSet')
        if self.uuid is not None:
            oprot.writeFieldBegin('uuid', TType.STRUCT, 1)
            self.uuid.write(oprot)
            oprot.writeFieldEnd()
        if self.metadata is not None:
            oprot.writeFieldBegin('metadata', TType.STRUCT, 2)
            self.metadata.write(oprot)
            oprot.writeFieldEnd()
        if self.mentionList is not None:
            oprot.writeFieldBegin('mentionList', TType.LIST, 3)
            oprot.writeListBegin(TType.STRUCT, len(self.mentionList))
            for iter47 in self.mentionList:
                iter47.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.linkingList is not None:
            oprot.writeFieldBegin('linkingList', TType.LIST, 4)
            oprot.writeListBegin(TType.STRUCT, len(self.linkingList))
            for iter48 in self.linkingList:
                iter48.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.uuid is None:
            raise TProtocolException(message='Required field uuid is unset!')
        if self.metadata is None:
            raise TProtocolException(message='Required field metadata is unset!')
        if self.mentionList is None:
            raise TProtocolException(message='Required field mentionList is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(Entity)
Entity.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'uuid', [concrete.uuid.ttypes.UUID, None], None, ),  # 1
    (2, TType.LIST, 'mentionIdList', (TType.STRUCT, [concrete.uuid.ttypes.UUID, None], False), None, ),  # 2
    (3, TType.STRING, 'type', 'UTF8', None, ),  # 3
    (4, TType.DOUBLE, 'confidence', None, None, ),  # 4
    (5, TType.STRING, 'canonicalName', 'UTF8', None, ),  # 5
    (6, TType.STRING, 'id', 'UTF8', None, ),  # 6
    (7, TType.LIST, 'rawMentionList', (TType.STRUCT, [concrete.structure.ttypes.TokenRefSequence, None], False), None, ),  # 7
)
all_structs.append(EntitySet)
EntitySet.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'uuid', [concrete.uuid.ttypes.UUID, None], None, ),  # 1
    (2, TType.STRUCT, 'metadata', [concrete.metadata.ttypes.AnnotationMetadata, None], None, ),  # 2
    (3, TType.LIST, 'entityList', (TType.STRUCT, [Entity, None], False), None, ),  # 3
    (4, TType.LIST, 'linkingList', (TType.STRUCT, [concrete.linking.ttypes.Linking, None], False), None, ),  # 4
    (5, TType.STRUCT, 'mentionSetId', [concrete.uuid.ttypes.UUID, None], None, ),  # 5
)
all_structs.append(EntityMention)
EntityMention.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'uuid', [concrete.uuid.ttypes.UUID, None], None, ),  # 1
    (2, TType.STRUCT, 'tokens', [concrete.structure.ttypes.TokenRefSequence, None], None, ),  # 2
    (3, TType.STRING, 'entityType', 'UTF8', None, ),  # 3
    (4, TType.STRING, 'phraseType', 'UTF8', None, ),  # 4
    (5, TType.DOUBLE, 'confidence', None, None, ),  # 5
    (6, TType.STRING, 'text', 'UTF8', None, ),  # 6
    (7, TType.LIST, 'childMentionIdList', (TType.STRUCT, [concrete.uuid.ttypes.UUID, None], False), None, ),  # 7
)
all_structs.append(EntityMentionSet)
EntityMentionSet.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'uuid', [concrete.uuid.ttypes.UUID, None], None, ),  # 1
    (2, TType.STRUCT, 'metadata', [concrete.metadata.ttypes.AnnotationMetadata, None], None, ),  # 2
    (3, TType.LIST, 'mentionList', (TType.STRUCT, [EntityMention, None], False), None, ),  # 3
    (4, TType.LIST, 'linkingList', (TType.STRUCT, [concrete.linking.ttypes.Linking, None], False), None, ),  # 4
)
fix_spec(all_structs)
del all_structs



"""Code for reading and writing Concrete Communications
"""
from __future__ import unicode_literals

try:
    from cStringIO import StringIO as BytesIO
except ImportError:
    from io import BytesIO

from gzip import open as gzip_open
import bz2
import mimetypes
import tarfile
import zipfile

import os
import time

from thrift import TSerialization
from thrift.transport import TTransport

from ..communication.ttypes import Communication
from ..structure.ttypes import TokenLattice
from .references import add_references_to_communication
from .thrift_factory import factory

try:
    unicode
except NameError:
    unicode = str


if os.name == 'nt':
    # Windows NT does not have os.{uid,gid,username,groupname} so we
    # define stubs.

    def _get_uid():
        return 0

    def _get_gid():
        return 0

    def _get_username():
        return ''

    def _get_groupname():
        return ''

else:
    import pwd
    import grp

    def _get_uid():
        return os.getuid()

    def _get_gid():
        return os.getgid()

    def _get_username():
        return pwd.getpwuid(_get_uid()).pw_name

    def _get_groupname():
        return grp.getgrgid(_get_gid()).gr_name


def read_thrift_from_file(thrift_obj, filename):
    """Instantiate Thrift object from contents of named file

    The Thrift file is assumed to be encoded using TCompactProtocol

    **WARNING** - Thrift deserialization tends to fail silently.  For
    example, the Thrift libraries will not complain if you try to
    deserialize data from the file `/dev/urandom`.

    Args:
        thrift_obj: A Thrift object (e.g. a Communication object)
        filename (str):  A filename string

    Returns:
        The Thrift object that was passed in as an argument
    """
    thrift_file = open(filename, "rb")
    thrift_bytes = thrift_file.read()
    TSerialization.deserialize(
        thrift_obj, thrift_bytes,
        protocol_factory=factory.protocolFactory)
    thrift_file.close()
    return thrift_obj


def read_communication_from_file(communication_filename, add_references=True):
    """Read a Communication from the file specified by filename

    Args:
        communication_filename (str): String with filename
        add_references (bool): If True, calls
           :func:`concrete.util.references.add_references_to_communication`
           on :class:`.Communication` read from file

    Returns:
        Communication: Communication read from file
    """
    comm = read_thrift_from_file(Communication(), communication_filename)
    if add_references:
        add_references_to_communication(comm)
    return comm


def read_tokenlattice_from_file(tokenlattice_filename):
    """Read a :class:`.TokenLattice` from a file

    Args:
        tokenlattice_filename (str): Name of file containing serialized
            :class:`.TokenLattice`

    Returns:
        TokenLattice: TokenLattice read from file
    """
    return read_thrift_from_file(TokenLattice(), tokenlattice_filename)


def write_communication_to_file(communication, communication_filename):
    """Write a :class:`.Communication` to a file

    Args:
        communication (Communication): communication to write
        communication_filename (str): path of file to write to
    """
    write_thrift_to_file(communication, communication_filename)


def write_thrift_to_file(thrift_obj, filename):
    """Write a Thrift object to a file

    Args:
        thrift_obj: Thrift object to write
        filename (str): path of file to write to
    """
    thrift_bytes = TSerialization.serialize(
        thrift_obj,
        protocol_factory=factory.protocolFactory)
    thrift_file = open(filename, "wb")
    thrift_file.write(thrift_bytes)
    thrift_file.close()


class _FileTypeClass(object):
    '''
    Enum-like object representing the set of known filetypes.
    Filetype integer IDs can be accessed directly (by retrieving
    the corresponding constant public member variable of this object)
    or can be looked up from their integer IDs or string names (using
    the `lookup` method).  A tuple of all filetype names is provided
    in the member variable `CHOICES`.  There should only be one member
    of this class.
    '''

    def __init__(self, *names):
        '''
        Initialize from given filetype names.  Each name will be
        associated with a unique integer id.

        Args:
            names: known filetype names (each of type str)

        Raises:
            ValueError: if a filetype name conflicts with a member
                variable/method name
        '''
        self.CHOICES = tuple(names)
        for (i, name) in enumerate(names):
            if name in ('CHOICES', '_normalize', 'lookup', 'add_argument'):
                raise ValueError('%s is an invalid filetype name' % name)
            setattr(self, self._normalize(name), i)

    @classmethod
    def _normalize(cls, name):
        '''
        Return upper-cased name with underscores instead of hyphens
        (suitable for use as a member variable name).

        Args:
            name (str): string filetype name
        '''
        return name.replace('-', '_').upper()

    def lookup(self, ft):
        '''
        Convenience method:
        Look up and return integer filetype corresponding to filetype
        name (or, if given the integer filetype, return that integer).

        Args:
            ft: filetype name (str) or integer value

        Returns:
            filetype (integer value) for ft

        Raises:
            ValueError: if ft is not a known filetype name or id
        '''

        if isinstance(ft, int):
            return ft
        elif isinstance(ft, str) or isinstance(ft, unicode):
            return getattr(self, self._normalize(ft))
        else:
            raise ValueError('unknown filetype %s' % str(ft))

    def add_argument(self, argparse_parser, flag='--input-filetype',
                     help_name='input file'):
        '''
        Add filetype argument to parser, return function that can be
        called on argparse namespace to retrieve argument value, e.g.,

        >>> get_filetype = add_argument(parser, ...)
        >>> ns = parser.parse_args()
        >>> filetype = get_filetype(ns)
        '''
        argparse_parser.add_argument(flag, type=str, choices=self.CHOICES,
                                     default=self.CHOICES[0],
                                     help='filetype for %s (choices: %s)' %
                                          (help_name, ', '.join(self.CHOICES)))

        def _get_value(ns):
            return getattr(ns, flag.lstrip('-').replace('-', '_'))
        return _get_value


FileType = _FileTypeClass(
    'auto',
    'zip',
    'tar', 'tar-gz', 'tar-bz2',
    'stream', 'stream-gz', 'stream-bz2',
)


class ThriftReader(object):
    """Iterator/generator class for reading one or more Thrift structures
    from a file

    The iterator returns a `(obj, filename)` tuple where obj is an object
    of type thrift_type.

    Supported filetypes are:

    - a file with a single Thrift structure
    - a file with multiple Thrift structures concatenated together
    - a gzipped file with a single Thrift structure
    - a gzipped file with multiple Thrift structures concatenated
      together
    - a .tar.gz file with one or more Thrift structures
    - a .zip file with one or more Thrift structures

    Sample usage::

        for (comm, filename) in ThriftReader(Communication,
                                             'multiple_comms.tar.gz'):
            do_something(comm)
    """

    def __init__(self, thrift_type, filename,
                 postprocess=None, filetype=FileType.AUTO):
        """
        Args:
            thrift_type: Class for Thrift type, e.g. Communication, TokenLattice
            filename (str):
            postprocess (function): A post-processing function that is called
                with the Thrift object as argument each time a Thrift object
                is read from the file
            filetype (FileType): Expected type of file.  Default value is
                `FileType.AUTO`, where function will try to automatically
                determine file type.

        Raises:
            ValueError: if filetype is not a known filetype name or id
        """
        filetype = FileType.lookup(filetype)

        self._seek_supported = True

        self._thrift_type = thrift_type
        if postprocess is None:
            def _noop(obj):
                return
            self._postprocess = _noop
        else:
            self._postprocess = postprocess
        self._source_filename = filename

        if filetype == FileType.TAR:
            self.filetype = 'tar'
            self.tar = tarfile.open(filename, 'r|')

        elif filetype == FileType.TAR_GZ:
            self.filetype = 'tar'
            self.tar = tarfile.open(filename, 'r|gz')

        elif filetype == FileType.TAR_BZ2:
            self.filetype = 'tar'
            self.tar = tarfile.open(filename, 'r|bz2')

        elif filetype == FileType.ZIP:
            self.filetype = 'zip'
            self.zip = zipfile.ZipFile(filename, 'r')
            self.zip_infolist = self.zip.infolist()
            self.zip_infolist_index = 0

        elif filetype == FileType.STREAM:
            self.filetype = 'stream'
            f = open(filename, 'rb')

        elif filetype == FileType.STREAM_GZ:
            self.filetype = 'stream'
            f = gzip_open(filename, 'rb')

        elif filetype == FileType.STREAM_BZ2:
            self.filetype = 'stream'
            f = bz2.BZ2File(filename, 'r')

        elif filetype == FileType.AUTO:
            if tarfile.is_tarfile(filename):
                self.filetype = 'tar'
                self.tar = tarfile.open(filename, 'r|*')

            elif zipfile.is_zipfile(filename):
                self.filetype = 'zip'
                self.zip = zipfile.ZipFile(filename, 'r')
                self.zip_infolist = self.zip.infolist()
                self.zip_infolist_index = 0

            elif mimetypes.guess_type(filename)[1] == 'gzip':
                # this is not a true stream---is_tarfile will have
                # successfully seeked backwards on the file if we have
                # reached this point
                self.filetype = 'stream'
                f = gzip_open(filename, 'rb')

            elif mimetypes.guess_type(filename)[1] == 'bzip2':
                # this is not a true stream
                self.filetype = 'stream'
                f = bz2.BZ2File(filename, 'r')

            else:
                # this is not a true stream
                self.filetype = 'stream'
                f = open(filename, 'rb')

        else:
            raise ValueError('unknown filetype %d' % filetype)

        if self.filetype == 'stream':
            self.transport = TTransport.TFileObjectTransport(f)
            self.protocol = factory.createProtocol(self.transport)
            self.transport.open()

    def __iter__(self):
        return self

    def __next__(self):
        """Returns a `(obj, filename)` tuple where obj is an object
        of type thrift_type.

        If the ThriftReader is reading from an archive, then
        `filename` will be set to the name of the Thrift entry in
        the archive (e.g. `foo.concrete`), and not the name of the
        archive file (e.g. `bar.zip`).  If the ThriftReader is reading
        from a concatenated file (instead of an archive), then all
        Thrift structures extracted from the concatenated file will have
        the same value for the `filename` field.

        Raises:
            ValueError: if self.filetype (normally validated by
            constructor) is not a known filetype name or id
        """
        if self.filetype == 'stream':
            return self._next_from_stream()
        elif self.filetype == 'tar':
            return self._next_from_tar()
        elif self.filetype == 'zip':
            return self._next_from_zip()
        else:
            raise ValueError('unknown filetype %s' % self.filetype)

    def next(self):
        '''
        Return tuple containing next communication (and filename)
        in the sequence.

        Raises:
            EOFError: unexpected EOF, probably caused by deserializing an invalid Thrift object
            StopIteration: if there are no more communications

        Returns:
            tuple containing Communication object and its filename
        '''
        return self.__next__()

    def _next_from_stream(self):
        '''
        Return tuple containing next Thrift object (and filename)
        from an uncompressed stream.

        Raises:
            EOFError: unexpected EOF, probably caused by deserializing an invalid Thrift object
            StopIteration: if there are no more Thrift objects

        Returns:
            tuple containing Thrift object and its filename
        '''
        if self.transport.fileobj and self._seek_supported:
            try:
                file_pos_0 = self.transport.fileobj.tell()
            except IOError as e:
                if e.errno == 29:  # Illegal seek
                    self._seek_supported = False
                else:
                    raise e
        try:
            thrift_obj = self._thrift_type()
            thrift_obj.read(self.protocol)
            self._postprocess(thrift_obj)
            return (thrift_obj, self._source_filename)
        except EOFError:
            if self.transport.fileobj and self._seek_supported:
                # If the file position moved after the read() call, we weren't truly
                # at the End Of File.
                file_pos = self.transport.fileobj.tell()
                if file_pos != file_pos_0:
                    self.transport.close()
                    raise EOFError(
                        'While trying to read Thrift object of type %s starting at byte %d. ' %
                        (type(self._thrift_type()), file_pos_0))
            self.transport.close()
            raise StopIteration

    def _next_from_tar(self):
        '''
        Return tuple containing next communication (and filename)
        from a tar file.

        Raises:
            StopIteration: if there are no more communications

        Returns:
            tuple containing Communication object and its filename
        '''
        while True:
            tarinfo = self.tar.next()
            if tarinfo is None:
                raise StopIteration
            if not tarinfo.isfile():
                # Ignore directories
                continue
            filename = os.path.split(tarinfo.name)[-1]
            if filename[0] == '.' and filename[1] == '_':
                # Ignore attribute files created by OS X tar
                continue
            comm = TSerialization.deserialize(
                self._thrift_type(),
                self.tar.extractfile(tarinfo).read(),
                protocol_factory=factory.protocolFactory)
            self._postprocess(comm)
            # hack to keep memory usage O(1)
            # (...but the real hack is tarfile :)
            self.tar.members = []
            return (comm, tarinfo.name)

    def _next_from_zip(self):
        '''
        Return tuple containing next communication (and filename)
        from a zip file.

        Raises:
            StopIteration: if there are no more communications

        Returns:
            tuple containing Communication object and its filename
        '''
        if self.zip_infolist_index >= len(self.zip_infolist):
            raise StopIteration
        zipinfo = self.zip_infolist[self.zip_infolist_index]
        self.zip_infolist_index += 1
        comm = TSerialization.deserialize(
            self._thrift_type(),
            self.zip.open(zipinfo).read(),
            protocol_factory=factory.protocolFactory)
        self._postprocess(comm)
        return (comm, zipinfo.filename)


class CommunicationReader(ThriftReader):
    """Iterator/generator class for reading one or more Communications from a
    file

    The iterator returns a `(Communication, filename)` tuple

    Supported filetypes are:

    - a file with a single Communication
    - a file with multiple Communications concatenated together
    - a gzipped file with a single Communication
    - a gzipped file with multiple Communications concatenated together
    - a .tar.gz file with one or more Communications
    - a .zip file with one or more Communications

    Sample usage::

        for (comm, filename) in CommunicationReader('multiple_comms.tar.gz'):
            do_something(comm)
    """

    def __init__(self, filename, add_references=True, filetype=FileType.AUTO):
        """
        Args:
            filename (str): path of file to read from
            add_references (bool): If True, calls
               :func:`concrete.util.references.add_references_to_communication`
               on all :class:`.Communication` objects read from file
            filetype (FileType): Expected type of file.  Default value is
                `FileType.AUTO`, where function will try to automatically
                determine file type.
        """
        super(CommunicationReader, self).__init__(
            Communication,
            filename,
            postprocess=(add_references_to_communication
                         if add_references
                         else None),
            filetype=filetype)


class CommunicationWriter(object):
    """Class for writing one or more Communications to a file

    Sample usage::

        with CommunicationWriter('foo.concrete') as writer:
            writer.write(existing_comm_object)
    """

    def __init__(self, filename=None, gzip=False):
        """
        Args:
            filename (str): if specified, open file at this path
                during construction (a file can alternatively be opened
                after construction using the open method)
            gzip (bool): Flag indicating if file should be
                compressed with gzip
        """
        self.gzip = gzip
        if filename is not None:
            self.open(filename)

    def close(self):
        '''
        Close file.
        '''
        self.file.close()

    def open(self, filename):
        """
        Open specified file for writing.  File will be compressed
        if the gzip flag of the constructor was set to True.

        Args:
            filename (str): path to file to open for writing
        """
        if self.gzip:
            self.file = gzip_open(filename, 'wb')
        else:
            self.file = open(filename, 'wb')

    def write(self, comm):
        """
        Args:
            comm (Communication): communication to write to file
        """
        thrift_bytes = TSerialization.serialize(
            comm, protocol_factory=factory.protocolFactory)
        self.file.write(thrift_bytes)

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self.close()


class CommunicationWriterTar(object):
    """Class for writing one or more Communications to a .tar archive

    Sample usage::

        with CommunicationWriterTar('multiple_comms.tar') as writer:
            writer.write(comm_object_one, 'comm_one.concrete')
            writer.write(comm_object_two, 'comm_two.concrete')
            writer.write(comm_object_three, 'comm_three.concrete')
    """

    def __init__(self, tar_filename=None, gzip=False):
        """
        Args:
            tar_filename (str): if specified, open file at this path
                during construction (a file can alternatively be opened
                after construction using the open method)
            gzip (bool): Flag indicating if .tar file should be
                compressed with gzip
        """
        self.gzip = gzip
        if tar_filename is not None:
            self.open(tar_filename)

    def close(self):
        '''
        Close tar file.
        '''
        self.tarfile.close()

    def open(self, tar_filename):
        """
        Open specified tar file for writing.  File will be compressed
        if the gzip flag of the constructor was set to True.

        Args:
            tar_filename (str): path to file to open for writing
        """
        self.tarfile = tarfile.open(tar_filename, 'w:gz' if self.gzip else 'w')

    def write(self, comm, comm_filename=None):
        """
        Args:
            comm (Communication): communication to write to tar file
            comm_filename (str): desired filename of communication
                within tar file (by default the filename will be the
                communication id appended with a .concrete extension)
        """
        if comm_filename is None:
            comm_filename = comm.id + '.concrete'

        thrift_bytes = TSerialization.serialize(
            comm, protocol_factory=factory.protocolFactory)

        file_like_obj = BytesIO(thrift_bytes)

        comm_tarinfo = tarfile.TarInfo()
        comm_tarinfo.type = tarfile.REGTYPE
        comm_tarinfo.name = comm_filename
        comm_tarinfo.size = len(thrift_bytes)
        comm_tarinfo.mode = 0o644
        comm_tarinfo.mtime = time.time()
        comm_tarinfo.uid = _get_uid()
        comm_tarinfo.uname = _get_username()
        comm_tarinfo.gid = _get_gid()
        comm_tarinfo.gname = _get_groupname()

        self.tarfile.addfile(comm_tarinfo, file_like_obj)

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self.close()


class CommunicationWriterTGZ(CommunicationWriterTar):
    """Class for writing one or more Communications to a .tar.gz (.tgz)
    archive

    Sample usage::

        with CommunicationWriterTGZ('multiple_comms.tar.gz') as writer:
            writer.write(comm_object_one, 'comm_one.concrete')
            writer.write(comm_object_two, 'comm_two.concrete')
            writer.write(comm_object_three, 'comm_three.concrete')
    """

    def __init__(self, tar_filename=None):
        super(CommunicationWriterTGZ, self).__init__(tar_filename, gzip=True)


class CommunicationWriterZip(object):
    '''Class for writing one or more Communications to a .zip archive

    Sample usage::

        with CommunicationWriterZip('multiple_comms.zip') as writer:
            writer.write(comm_object_one, 'comm_one.concrete')
            writer.write(comm_object_two, 'comm_two.concrete')
            writer.write(comm_object_three, 'comm_three.concrete')
    '''

    def __init__(self, zip_filename=None):
        '''
        Args:
            zip_filename (str): if specified, open file at this path
                during construction (a file can alternatively be opened
                after construction using the open method)
        '''
        if zip_filename is not None:
            self.open(zip_filename)

    def open(self, zip_filename=None):
        '''
        Open specified zip file for writing.

        Args:
            zip_filename (str): path to file to open for writing
        '''
        self.zip_f = zipfile.ZipFile(zip_filename, 'w')

    def close(self):
        '''
        Close zip file.
        '''
        self.zip_f.close()

    def write(self, comm, comm_filename=None):
        '''
        Write communication to zip file.'

        Args:
            comm (Communication): communication to write to zip file
            comm_filename (str): desired filename of communication
                within zip file (by default the filename will be the
                communication id appended with a .concrete extension)
        '''
        if comm_filename is None:
            comm_filename = comm.id + '.concrete'

        thrift_bytes = TSerialization.serialize(
            comm, protocol_factory=factory.protocolFactory)

        self.zip_f.writestr(comm_filename, thrift_bytes)

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self.close()



from ..access import (
    FetchCommunicationService,
    StoreCommunicationService)
from .service_wrapper import (
    ConcreteServiceClientWrapper,
    ConcreteServiceWrapper,
    HTTPConcreteServiceClientWrapper,
    SubprocessConcreteServiceWrapper)


FetchCommunicationClientWrapper = type(
    'FetchCommunicationClientWrapper',
    (ConcreteServiceClientWrapper,),
    {'concrete_service_class': FetchCommunicationService})


FetchCommunicationServiceWrapper = type(
    'FetchCommunicationServiceWrapper',
    (ConcreteServiceWrapper,),
    {'concrete_service_class': FetchCommunicationService})


SubprocessFetchCommunicationServiceWrapper = type(
    'SubprocessFetchCommunicationServiceWrapper',
    (SubprocessConcreteServiceWrapper,),
    {'concrete_service_wrapper_class': FetchCommunicationServiceWrapper})


HTTPFetchCommunicationClientWrapper = type(
    'HTTPFetchCommunicationClientWrapper',
    (HTTPConcreteServiceClientWrapper,),
    {'concrete_service_class': FetchCommunicationService})


HTTPStoreCommunicationClientWrapper = type(
    'HTTPStoreCommunicationClientWrapper',
    (HTTPConcreteServiceClientWrapper,),
    {'concrete_service_class': StoreCommunicationService})


StoreCommunicationClientWrapper = type(
    'StoreCommunicationClientWrapper',
    (ConcreteServiceClientWrapper,),
    {'concrete_service_class': StoreCommunicationService})


StoreCommunicationServiceWrapper = type(
    'StoreCommunicationServiceWrapper',
    (ConcreteServiceWrapper,),
    {'concrete_service_class': StoreCommunicationService})


SubprocessStoreCommunicationServiceWrapper = type(
    'SubprocessStoreCommunicationServiceWrapper',
    (SubprocessConcreteServiceWrapper,),
    {'concrete_service_wrapper_class': StoreCommunicationServiceWrapper})



# -*- coding: utf-8 -*-
#
# Autogenerated by Thrift Compiler (0.11.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:coding=utf-8
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec

import sys
import concrete.metadata.ttypes
import concrete.uuid.ttypes

from thrift.transport import TTransport
all_structs = []


class LanguageIdentification(object):
    """
    A theory about what languages are present in a given communication
    or piece of communication.  Note that it is possible to have more
    than one language present in a given communication.

    Attributes:
     - uuid: Unique identifier for this language identification.
     - metadata: Information about where this language identification came from.
     - languageToProbabilityMap: A list mapping from a language to the probability that that
    language occurs in a given communication.  Each language code should
    occur at most once in this list.  The probabilities do <i>not</i>
    need to sum to one -- for example, if a single communication is known
    to contain both English and French, then it would be appropriate
    to assign a probability of 1 to both langauges.  (Manually
    annotated LanguageProb objects should always have probabilities
    of either zero or one; machine-generated LanguageProbs may have
    intermediate probabilities.)

    Note: The string key should represent the ISO 639-3 three-letter code.
    """


    def __init__(self, uuid=None, metadata=None, languageToProbabilityMap=None,):
        self.uuid = uuid
        self.metadata = metadata
        self.languageToProbabilityMap = languageToProbabilityMap

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.uuid = concrete.uuid.ttypes.UUID()
                    self.uuid.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRUCT:
                    self.metadata = concrete.metadata.ttypes.AnnotationMetadata()
                    self.metadata.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.MAP:
                    self.languageToProbabilityMap = {}
                    (_ktype1, _vtype2, _size0) = iprot.readMapBegin()
                    for _i4 in range(_size0):
                        _key5 = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                        _val6 = iprot.readDouble()
                        self.languageToProbabilityMap[_key5] = _val6
                    iprot.readMapEnd()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('LanguageIdentification')
        if self.uuid is not None:
            oprot.writeFieldBegin('uuid', TType.STRUCT, 1)
            self.uuid.write(oprot)
            oprot.writeFieldEnd()
        if self.metadata is not None:
            oprot.writeFieldBegin('metadata', TType.STRUCT, 2)
            self.metadata.write(oprot)
            oprot.writeFieldEnd()
        if self.languageToProbabilityMap is not None:
            oprot.writeFieldBegin('languageToProbabilityMap', TType.MAP, 3)
            oprot.writeMapBegin(TType.STRING, TType.DOUBLE, len(self.languageToProbabilityMap))
            for kiter7, viter8 in self.languageToProbabilityMap.items():
                oprot.writeString(kiter7.encode('utf-8') if sys.version_info[0] == 2 else kiter7)
                oprot.writeDouble(viter8)
            oprot.writeMapEnd()
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.uuid is None:
            raise TProtocolException(message='Required field uuid is unset!')
        if self.metadata is None:
            raise TProtocolException(message='Required field metadata is unset!')
        if self.languageToProbabilityMap is None:
            raise TProtocolException(message='Required field languageToProbabilityMap is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(LanguageIdentification)
LanguageIdentification.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'uuid', [concrete.uuid.ttypes.UUID, None], None, ),  # 1
    (2, TType.STRUCT, 'metadata', [concrete.metadata.ttypes.AnnotationMetadata, None], None, ),  # 2
    (3, TType.MAP, 'languageToProbabilityMap', (TType.STRING, 'UTF8', TType.DOUBLE, None, False), None, ),  # 3
)
fix_spec(all_structs)
del all_structs



#!/usr/bin/env python

from __future__ import print_function
from __future__ import unicode_literals

import argparse
import logging

from boto import connect_s3
import concrete.version
from concrete.util import (
    set_stdout_encoding,
    StoreCommunicationServiceWrapper,
    S3BackedStoreHandler,
    DEFAULT_S3_KEY_PREFIX_LEN,
)


def main():
    set_stdout_encoding()

    parser = argparse.ArgumentParser(
        description='Run a store server backed by an AWS S3 bucket '
                    '(keyed by communication id).',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    parser.add_argument('bucket_name', help='name of S3 bucket to write to')
    parser.add_argument('--host', help='hostname to serve on',
                        default='localhost')
    parser.add_argument('--port', type=int, help='port to serve on',
                        default=9090)
    parser.add_argument('--prefix-len', type=int, default=DEFAULT_S3_KEY_PREFIX_LEN,
                        help='S3 keys are prefixed with hashes of this length')
    parser.add_argument('-l', '--loglevel', '--log-level',
                        help='Logging verbosity level threshold (to stderr)',
                        default='info')
    concrete.version.add_argparse_argument(parser)
    args = parser.parse_args()

    logging.basicConfig(format='%(asctime)-15s %(levelname)s: %(message)s',
                        level=args.loglevel.upper())

    logging.info('connecting to s3')
    conn = connect_s3()
    logging.info('retrieving bucket {}'.format(args.bucket_name))
    bucket = conn.get_bucket(args.bucket_name)
    logging.info('writing to s3 bucket {}, prefix length {}'.format(
        args.bucket_name, args.prefix_len))
    handler = S3BackedStoreHandler(bucket, args.prefix_len)
    logging.info('hosting store service at {}:{}'.format(args.host, args.port))
    server = StoreCommunicationServiceWrapper(handler)
    server.serve(args.host, args.port)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
#
# Autogenerated by Thrift Compiler (0.11.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:coding=utf-8
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec

import sys
import concrete.communication.ttypes
import concrete.services.ttypes
import concrete.structure.ttypes
import concrete.uuid.ttypes
import concrete.metadata.ttypes
import concrete.entities.ttypes

from thrift.transport import TTransport
all_structs = []


class SearchType(object):
    """
    What are we searching over
    """
    COMMUNICATIONS = 0
    SECTIONS = 1
    SENTENCES = 2
    ENTITIES = 3
    ENTITY_MENTIONS = 4
    SITUATIONS = 5
    SITUATION_MENTIONS = 6

    _VALUES_TO_NAMES = {
        0: "COMMUNICATIONS",
        1: "SECTIONS",
        2: "SENTENCES",
        3: "ENTITIES",
        4: "ENTITY_MENTIONS",
        5: "SITUATIONS",
        6: "SITUATION_MENTIONS",
    }

    _NAMES_TO_VALUES = {
        "COMMUNICATIONS": 0,
        "SECTIONS": 1,
        "SENTENCES": 2,
        "ENTITIES": 3,
        "ENTITY_MENTIONS": 4,
        "SITUATIONS": 5,
        "SITUATION_MENTIONS": 6,
    }


class SearchFeedback(object):
    """
    Feedback values
    """
    NEGATIVE = -1
    NONE = 0
    POSITIVE = 1

    _VALUES_TO_NAMES = {
        -1: "NEGATIVE",
        0: "NONE",
        1: "POSITIVE",
    }

    _NAMES_TO_VALUES = {
        "NEGATIVE": -1,
        "NONE": 0,
        "POSITIVE": 1,
    }


class SearchCapability(object):
    """
    A search provider describes its capabilities with a list of search type and language pairs.

    Attributes:
     - type: A type of search supported by the search provider
     - lang: Language that the search provider supports.
    Use ISO 639-2/T three letter codes.
    """


    def __init__(self, type=None, lang=None,):
        self.type = type
        self.lang = lang

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.I32:
                    self.type = iprot.readI32()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRING:
                    self.lang = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('SearchCapability')
        if self.type is not None:
            oprot.writeFieldBegin('type', TType.I32, 1)
            oprot.writeI32(self.type)
            oprot.writeFieldEnd()
        if self.lang is not None:
            oprot.writeFieldBegin('lang', TType.STRING, 2)
            oprot.writeString(self.lang.encode('utf-8') if sys.version_info[0] == 2 else self.lang)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.type is None:
            raise TProtocolException(message='Required field type is unset!')
        if self.lang is None:
            raise TProtocolException(message='Required field lang is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class SearchQuery(object):
    """
    Wrapper for information relevant to a (possibly structured) search.

    Attributes:
     - terms: Individual words, or multiword phrases, e.g., 'dog', 'blue
    cheese'.  It is the responsibility of the implementation of
    Search* to tokenize multiword phrases, if so-desired.  Further,
    an implementation may choose to support advanced features such as
    wildcards, e.g.: 'blue*'.  This specification makes no
    committment as to the internal structure of keywords and their
    semantics: that is the responsibility of the individual
    implementation.
     - questions: e.g., "what is the capital of spain?"

    questions is a list in order that possibly different phrasings of
    the question can be included, e.g.: "what is the name of spain's
    capital?"
     - communicationId: Refers to an optional communication that can provide context for the query.
     - tokens: Refers to a sequence of tokens in the communication referenced by communicationId.
     - rawQuery: The input from the user provided in the search box, unmodified
     - auths: optional authorization mechanism
     - userId: Identifies the user who submitted the search query
     - name: Human readable name of the query.
     - labels: Properties of the query or user.
    These labels can be used to group queries and results by a domain or group of
    users for training. An example usage would be assigning the geographical region
    as a label ("spain"). User labels could be based on organizational units ("hltcoe").
     - type: This search is over this type of data (communications, sentences, entities)
     - lang: The language of the corpus that the user wants to search.
    Use ISO 639-2/T three letter codes.
     - corpus: An identifier of the corpus that the search is to be performed over.
     - k: The maximum number of candidates the search service should return.
     - communication: An optional communication used as context for the query.
    If both this field and communicationId is populated, then it is
    assumed the ID of the communication is the same as communicationId.
    """


    def __init__(self, terms=None, questions=None, communicationId=None, tokens=None, rawQuery=None, auths=None, userId=None, name=None, labels=None, type=None, lang=None, corpus=None, k=None, communication=None,):
        self.terms = terms
        self.questions = questions
        self.communicationId = communicationId
        self.tokens = tokens
        self.rawQuery = rawQuery
        self.auths = auths
        self.userId = userId
        self.name = name
        self.labels = labels
        self.type = type
        self.lang = lang
        self.corpus = corpus
        self.k = k
        self.communication = communication

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.LIST:
                    self.terms = []
                    (_etype3, _size0) = iprot.readListBegin()
                    for _i4 in range(_size0):
                        _elem5 = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                        self.terms.append(_elem5)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.LIST:
                    self.questions = []
                    (_etype9, _size6) = iprot.readListBegin()
                    for _i10 in range(_size6):
                        _elem11 = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                        self.questions.append(_elem11)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.STRING:
                    self.communicationId = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.STRUCT:
                    self.tokens = concrete.structure.ttypes.TokenRefSequence()
                    self.tokens.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 5:
                if ftype == TType.STRING:
                    self.rawQuery = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 6:
                if ftype == TType.STRING:
                    self.auths = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 7:
                if ftype == TType.STRING:
                    self.userId = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 8:
                if ftype == TType.STRING:
                    self.name = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 9:
                if ftype == TType.LIST:
                    self.labels = []
                    (_etype15, _size12) = iprot.readListBegin()
                    for _i16 in range(_size12):
                        _elem17 = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                        self.labels.append(_elem17)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 10:
                if ftype == TType.I32:
                    self.type = iprot.readI32()
                else:
                    iprot.skip(ftype)
            elif fid == 11:
                if ftype == TType.STRING:
                    self.lang = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 12:
                if ftype == TType.STRING:
                    self.corpus = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 13:
                if ftype == TType.I32:
                    self.k = iprot.readI32()
                else:
                    iprot.skip(ftype)
            elif fid == 14:
                if ftype == TType.STRUCT:
                    self.communication = concrete.communication.ttypes.Communication()
                    self.communication.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('SearchQuery')
        if self.terms is not None:
            oprot.writeFieldBegin('terms', TType.LIST, 1)
            oprot.writeListBegin(TType.STRING, len(self.terms))
            for iter18 in self.terms:
                oprot.writeString(iter18.encode('utf-8') if sys.version_info[0] == 2 else iter18)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.questions is not None:
            oprot.writeFieldBegin('questions', TType.LIST, 2)
            oprot.writeListBegin(TType.STRING, len(self.questions))
            for iter19 in self.questions:
                oprot.writeString(iter19.encode('utf-8') if sys.version_info[0] == 2 else iter19)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.communicationId is not None:
            oprot.writeFieldBegin('communicationId', TType.STRING, 3)
            oprot.writeString(self.communicationId.encode('utf-8') if sys.version_info[0] == 2 else self.communicationId)
            oprot.writeFieldEnd()
        if self.tokens is not None:
            oprot.writeFieldBegin('tokens', TType.STRUCT, 4)
            self.tokens.write(oprot)
            oprot.writeFieldEnd()
        if self.rawQuery is not None:
            oprot.writeFieldBegin('rawQuery', TType.STRING, 5)
            oprot.writeString(self.rawQuery.encode('utf-8') if sys.version_info[0] == 2 else self.rawQuery)
            oprot.writeFieldEnd()
        if self.auths is not None:
            oprot.writeFieldBegin('auths', TType.STRING, 6)
            oprot.writeString(self.auths.encode('utf-8') if sys.version_info[0] == 2 else self.auths)
            oprot.writeFieldEnd()
        if self.userId is not None:
            oprot.writeFieldBegin('userId', TType.STRING, 7)
            oprot.writeString(self.userId.encode('utf-8') if sys.version_info[0] == 2 else self.userId)
            oprot.writeFieldEnd()
        if self.name is not None:
            oprot.writeFieldBegin('name', TType.STRING, 8)
            oprot.writeString(self.name.encode('utf-8') if sys.version_info[0] == 2 else self.name)
            oprot.writeFieldEnd()
        if self.labels is not None:
            oprot.writeFieldBegin('labels', TType.LIST, 9)
            oprot.writeListBegin(TType.STRING, len(self.labels))
            for iter20 in self.labels:
                oprot.writeString(iter20.encode('utf-8') if sys.version_info[0] == 2 else iter20)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.type is not None:
            oprot.writeFieldBegin('type', TType.I32, 10)
            oprot.writeI32(self.type)
            oprot.writeFieldEnd()
        if self.lang is not None:
            oprot.writeFieldBegin('lang', TType.STRING, 11)
            oprot.writeString(self.lang.encode('utf-8') if sys.version_info[0] == 2 else self.lang)
            oprot.writeFieldEnd()
        if self.corpus is not None:
            oprot.writeFieldBegin('corpus', TType.STRING, 12)
            oprot.writeString(self.corpus.encode('utf-8') if sys.version_info[0] == 2 else self.corpus)
            oprot.writeFieldEnd()
        if self.k is not None:
            oprot.writeFieldBegin('k', TType.I32, 13)
            oprot.writeI32(self.k)
            oprot.writeFieldEnd()
        if self.communication is not None:
            oprot.writeFieldBegin('communication', TType.STRUCT, 14)
            self.communication.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.type is None:
            raise TProtocolException(message='Required field type is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class SearchResultItem(object):
    """
    An individual element returned from a search.  Most/all methods
    will return a communicationId, possibly with an associated score.
    For example if the target element type of the search is Sentence
    then the sentenceId field should be populated.

    Attributes:
     - communicationId
     - sentenceId: The UUID of the returned sentence, which appears in the
    communication referenced by communicationId.
     - score: Values are not restricted in range (e.g., do not have to be
    within [0,1]).  Higher is better.

     - tokens: If SearchType=ENTITY_MENTIONS then this field should be populated.
    Otherwise, this field may be optionally populated in order to
    provide a hint to the client as to where to center a
    visualization, or the extraction of context, etc.
     - entity: If SearchType=ENTITIES then this field should be populated.
    """


    def __init__(self, communicationId=None, sentenceId=None, score=None, tokens=None, entity=None,):
        self.communicationId = communicationId
        self.sentenceId = sentenceId
        self.score = score
        self.tokens = tokens
        self.entity = entity

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRING:
                    self.communicationId = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRUCT:
                    self.sentenceId = concrete.uuid.ttypes.UUID()
                    self.sentenceId.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.DOUBLE:
                    self.score = iprot.readDouble()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.STRUCT:
                    self.tokens = concrete.structure.ttypes.TokenRefSequence()
                    self.tokens.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 5:
                if ftype == TType.STRUCT:
                    self.entity = concrete.entities.ttypes.Entity()
                    self.entity.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('SearchResultItem')
        if self.communicationId is not None:
            oprot.writeFieldBegin('communicationId', TType.STRING, 1)
            oprot.writeString(self.communicationId.encode('utf-8') if sys.version_info[0] == 2 else self.communicationId)
            oprot.writeFieldEnd()
        if self.sentenceId is not None:
            oprot.writeFieldBegin('sentenceId', TType.STRUCT, 2)
            self.sentenceId.write(oprot)
            oprot.writeFieldEnd()
        if self.score is not None:
            oprot.writeFieldBegin('score', TType.DOUBLE, 3)
            oprot.writeDouble(self.score)
            oprot.writeFieldEnd()
        if self.tokens is not None:
            oprot.writeFieldBegin('tokens', TType.STRUCT, 4)
            self.tokens.write(oprot)
            oprot.writeFieldEnd()
        if self.entity is not None:
            oprot.writeFieldBegin('entity', TType.STRUCT, 5)
            self.entity.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class SearchResult(object):
    """
    Single wrapper for results from all the various Search* services.

    Attributes:
     - uuid: Unique identifier for the results of this search.
     - searchQuery: The query that led to this result.
    Useful for capturing feedback or building training data.
     - searchResultItems: The list is assumed sorted best to worst, which should be
    reflected by the values contained in the score field of each
    SearchResult, if that field is populated.
     - metadata: The system that provided the response: likely use case for
    populating this field is for building training data.  Presumably
    a system will not need/want to return this object in live use.
     - lang: The dominant language of the search results.
    Use ISO 639-2/T three letter codes.
    Search providers should set this when possible to support downstream processing.
    Do not set if it is not known.
    If multilingual, use the string "multilingual".
    """


    def __init__(self, uuid=None, searchQuery=None, searchResultItems=None, metadata=None, lang=None,):
        self.uuid = uuid
        self.searchQuery = searchQuery
        self.searchResultItems = searchResultItems
        self.metadata = metadata
        self.lang = lang

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.uuid = concrete.uuid.ttypes.UUID()
                    self.uuid.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRUCT:
                    self.searchQuery = SearchQuery()
                    self.searchQuery.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.LIST:
                    self.searchResultItems = []
                    (_etype24, _size21) = iprot.readListBegin()
                    for _i25 in range(_size21):
                        _elem26 = SearchResultItem()
                        _elem26.read(iprot)
                        self.searchResultItems.append(_elem26)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.STRUCT:
                    self.metadata = concrete.metadata.ttypes.AnnotationMetadata()
                    self.metadata.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 5:
                if ftype == TType.STRING:
                    self.lang = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('SearchResult')
        if self.uuid is not None:
            oprot.writeFieldBegin('uuid', TType.STRUCT, 1)
            self.uuid.write(oprot)
            oprot.writeFieldEnd()
        if self.searchQuery is not None:
            oprot.writeFieldBegin('searchQuery', TType.STRUCT, 2)
            self.searchQuery.write(oprot)
            oprot.writeFieldEnd()
        if self.searchResultItems is not None:
            oprot.writeFieldBegin('searchResultItems', TType.LIST, 3)
            oprot.writeListBegin(TType.STRUCT, len(self.searchResultItems))
            for iter27 in self.searchResultItems:
                iter27.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.metadata is not None:
            oprot.writeFieldBegin('metadata', TType.STRUCT, 4)
            self.metadata.write(oprot)
            oprot.writeFieldEnd()
        if self.lang is not None:
            oprot.writeFieldBegin('lang', TType.STRING, 5)
            oprot.writeString(self.lang.encode('utf-8') if sys.version_info[0] == 2 else self.lang)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.uuid is None:
            raise TProtocolException(message='Required field uuid is unset!')
        if self.searchQuery is None:
            raise TProtocolException(message='Required field searchQuery is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(SearchCapability)
SearchCapability.thrift_spec = (
    None,  # 0
    (1, TType.I32, 'type', None, None, ),  # 1
    (2, TType.STRING, 'lang', 'UTF8', None, ),  # 2
)
all_structs.append(SearchQuery)
SearchQuery.thrift_spec = (
    None,  # 0
    (1, TType.LIST, 'terms', (TType.STRING, 'UTF8', False), None, ),  # 1
    (2, TType.LIST, 'questions', (TType.STRING, 'UTF8', False), None, ),  # 2
    (3, TType.STRING, 'communicationId', 'UTF8', None, ),  # 3
    (4, TType.STRUCT, 'tokens', [concrete.structure.ttypes.TokenRefSequence, None], None, ),  # 4
    (5, TType.STRING, 'rawQuery', 'UTF8', None, ),  # 5
    (6, TType.STRING, 'auths', 'UTF8', None, ),  # 6
    (7, TType.STRING, 'userId', 'UTF8', None, ),  # 7
    (8, TType.STRING, 'name', 'UTF8', None, ),  # 8
    (9, TType.LIST, 'labels', (TType.STRING, 'UTF8', False), None, ),  # 9
    (10, TType.I32, 'type', None, None, ),  # 10
    (11, TType.STRING, 'lang', 'UTF8', None, ),  # 11
    (12, TType.STRING, 'corpus', 'UTF8', None, ),  # 12
    (13, TType.I32, 'k', None, None, ),  # 13
    (14, TType.STRUCT, 'communication', [concrete.communication.ttypes.Communication, None], None, ),  # 14
)
all_structs.append(SearchResultItem)
SearchResultItem.thrift_spec = (
    None,  # 0
    (1, TType.STRING, 'communicationId', 'UTF8', None, ),  # 1
    (2, TType.STRUCT, 'sentenceId', [concrete.uuid.ttypes.UUID, None], None, ),  # 2
    (3, TType.DOUBLE, 'score', None, None, ),  # 3
    (4, TType.STRUCT, 'tokens', [concrete.structure.ttypes.TokenRefSequence, None], None, ),  # 4
    (5, TType.STRUCT, 'entity', [concrete.entities.ttypes.Entity, None], None, ),  # 5
)
all_structs.append(SearchResult)
SearchResult.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'uuid', [concrete.uuid.ttypes.UUID, None], None, ),  # 1
    (2, TType.STRUCT, 'searchQuery', [SearchQuery, None], None, ),  # 2
    (3, TType.LIST, 'searchResultItems', (TType.STRUCT, [SearchResultItem, None], False), None, ),  # 3
    (4, TType.STRUCT, 'metadata', [concrete.metadata.ttypes.AnnotationMetadata, None], None, ),  # 4
    (5, TType.STRING, 'lang', 'UTF8', None, ),  # 5
)
fix_spec(all_structs)
del all_structs



# -*- coding: utf-8 -*-
#
# Autogenerated by Thrift Compiler (0.11.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:coding=utf-8
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec

import sys
import concrete.communication.ttypes
import concrete.services.ttypes

from thrift.transport import TTransport
all_structs = []


class FetchResult(object):
    """
    Struct containing Communications from the FetchCommunicationService service.

    Attributes:
     - communications: a list of Communication objects that represent the results of the request
    """


    def __init__(self, communications=None,):
        self.communications = communications

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.LIST:
                    self.communications = []
                    (_etype3, _size0) = iprot.readListBegin()
                    for _i4 in range(_size0):
                        _elem5 = concrete.communication.ttypes.Communication()
                        _elem5.read(iprot)
                        self.communications.append(_elem5)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('FetchResult')
        if self.communications is not None:
            oprot.writeFieldBegin('communications', TType.LIST, 1)
            oprot.writeListBegin(TType.STRUCT, len(self.communications))
            for iter6 in self.communications:
                iter6.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.communications is None:
            raise TProtocolException(message='Required field communications is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class FetchRequest(object):
    """
    Struct representing a request for FetchCommunicationService.

    Attributes:
     - communicationIds: a list of Communication IDs
     - auths: optional authorization mechanism
    """


    def __init__(self, communicationIds=None, auths=None,):
        self.communicationIds = communicationIds
        self.auths = auths

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.LIST:
                    self.communicationIds = []
                    (_etype10, _size7) = iprot.readListBegin()
                    for _i11 in range(_size7):
                        _elem12 = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                        self.communicationIds.append(_elem12)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRING:
                    self.auths = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('FetchRequest')
        if self.communicationIds is not None:
            oprot.writeFieldBegin('communicationIds', TType.LIST, 1)
            oprot.writeListBegin(TType.STRING, len(self.communicationIds))
            for iter13 in self.communicationIds:
                oprot.writeString(iter13.encode('utf-8') if sys.version_info[0] == 2 else iter13)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.auths is not None:
            oprot.writeFieldBegin('auths', TType.STRING, 2)
            oprot.writeString(self.auths.encode('utf-8') if sys.version_info[0] == 2 else self.auths)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.communicationIds is None:
            raise TProtocolException(message='Required field communicationIds is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(FetchResult)
FetchResult.thrift_spec = (
    None,  # 0
    (1, TType.LIST, 'communications', (TType.STRUCT, [concrete.communication.ttypes.Communication, None], False), None, ),  # 1
)
all_structs.append(FetchRequest)
FetchRequest.thrift_spec = (
    None,  # 0
    (1, TType.LIST, 'communicationIds', (TType.STRING, 'UTF8', False), None, ),  # 1
    (2, TType.STRING, 'auths', 'UTF8', None, ),  # 2
)
fix_spec(all_structs)
del all_structs



#!/usr/bin/env python

"""Command line client for FetchCommunicationService
"""
from __future__ import print_function
from __future__ import unicode_literals

import argparse
import sys
import logging

import concrete.version
from concrete.access.ttypes import FetchRequest
from concrete.util import set_stdout_encoding
from concrete.util.access_wrapper import (
    FetchCommunicationClientWrapper,
    HTTPFetchCommunicationClientWrapper
)
from concrete.util.file_io import CommunicationWriterTGZ


def main():
    set_stdout_encoding()

    parser = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
        description="Interface with a Concrete FetchCommunicationService server. "
        "Supports either THttp/TJSONProtocol (using the '--uri' flag) "
        "or TSocket/TCompactProtocol (using '--host'/'--port')"
    )
    parser.add_argument("--host", default="localhost",
                        help="Hostname of TSocket/TCompactProtocol FetchCommunicationService")
    parser.add_argument("-p", "--port", type=int, default=9090,
                        help="Port of TSocket/TCompactProtocol FetchCommunicationService")
    parser.add_argument('--uri', '--url',
                        help="URI of THttpServer/TJSONProtocol FetchCommunicationService")
    parser.add_argument("--about", action="store_true",
                        help="Print value of fetch_service.about()")
    parser.add_argument("--alive", action="store_true",
                        help="Print value of fetch_service.alive()")
    parser.add_argument("--count", action="store_true",
                        help="Print value of fetch_service.getCommunicationCount()")
    parser.add_argument("--get-ids", action="store_true",
                        help="Print list of Communication IDs returned by "
                        "fetch_service.getCommunicationIDs(offset, count).  "
                        "The offset and count parameters are set using the "
                        "'--get-ids-offset' and '--get-ids-count' flags")
    parser.add_argument("--get-ids-offset", type=int, default=0, metavar="ID_OFFSET",
                        help="Number of Communication IDs printed using the '--get-ids' flag")
    parser.add_argument("--get-ids-count", type=int, default=20, metavar="ID_COUNT",
                        help="Offset for Communication IDs printed using the '--get-ids' flag")
    parser.add_argument("--save-as-tgz", metavar="TGZ_FILENAME",
                        help="Save fetched Communications to a TGZ archive containing files "
                        "named '[COMMUNICATION_ID].concrete'")
    parser.add_argument('-l', '--loglevel', '--log-level',
                        help='Logging verbosity level threshold (to stderr)',
                        default='info')
    parser.add_argument("comm_id", nargs="*", help="IDs of Communications to be fetched. "
                        "If '-' is specified, a list of Communication IDs will be read from "
                        "stdin, one Communication ID per line")
    concrete.version.add_argparse_argument(parser)
    args = parser.parse_args()

    logging.basicConfig(format='%(asctime)-15s %(levelname)s: %(message)s',
                        level=args.loglevel.upper())

    if args.uri:
        fetch_wrapper = HTTPFetchCommunicationClientWrapper(args.uri)
    else:
        fetch_wrapper = FetchCommunicationClientWrapper(args.host, args.port)

    with fetch_wrapper as client:
        if args.comm_id:
            fetch_request = FetchRequest()
            if len(args.comm_id) == 1 and args.comm_id[0] == '-':
                fetch_request.communicationIds = [line.strip() for line in sys.stdin.readlines()]
            else:
                fetch_request.communicationIds = args.comm_id
            fetch_result = client.fetch(fetch_request)
            print("Received FetchResult: '%s'" % fetch_result)

        if args.about:
            print("FetchCommunicationService.about() returned %s" % client.about())
        if args.alive:
            print("FetchCommunicationService.alive() returned %s" % client.alive())
        if args.count:
            print("FetchCommunicationService.getCommunicationCount() returned %d" %
                  client.getCommunicationCount())
        if args.get_ids:
            print("FetchCommunicationService.getCommunicationIDs(offset=%d, count=%d) returned:" %
                  (args.get_ids_offset, args.get_ids_count))
            for comm_id in client.getCommunicationIDs(args.get_ids_offset, args.get_ids_count):
                print("  %s" % comm_id)

        if args.save_as_tgz and args.comm_id:
            if fetch_result.communications:
                with CommunicationWriterTGZ(args.save_as_tgz) as writer:
                    for comm in fetch_result.communications:
                        comm_filename = '%s.concrete' % comm.id
                        print("Saving Communication to TGZ archive '%s' as '%s'" %
                              (args.save_as_tgz, comm_filename))
                        writer.write(comm, comm_filename)


if __name__ == '__main__':
    main()



from concrete.util import lun, dun, sun


def test_lun():
    assert lun([]) == []
    assert lun([3]) == [3]
    assert lun(None) == []


def test_dun():
    assert dun(dict()) == dict()
    assert dun(dict(x=3)) == dict(x=3)
    assert dun(None) == dict()


def test_sun():
    assert sun(set([])) == set([])
    assert sun(set([3])) == set([3])
    assert sun(None) == set([])



from ..search import SearchService, SearchProxyService, FeedbackService
from .service_wrapper import (
    ConcreteServiceClientWrapper,
    ConcreteServiceWrapper,
    HTTPConcreteServiceClientWrapper,
    SubprocessConcreteServiceWrapper)


HTTPSearchClientWrapper = type(
    'HTTPSearchClientWrapper',
    (HTTPConcreteServiceClientWrapper,),
    {'concrete_service_class': SearchService})


SearchClientWrapper = type(
    'SearchClientWrapper',
    (ConcreteServiceClientWrapper,),
    {'concrete_service_class': SearchService})


SearchServiceWrapper = type(
    'SearchServiceWrapper',
    (ConcreteServiceWrapper,),
    {'concrete_service_class': SearchService})


SubprocessSearchServiceWrapper = type(
    'SubprocessSearchServiceWrapper',
    (SubprocessConcreteServiceWrapper,),
    {'concrete_service_wrapper_class': SearchServiceWrapper})


SearchProxyClientWrapper = type(
    'SearchProxyClientWrapper',
    (ConcreteServiceClientWrapper,),
    {'concrete_service_class': SearchProxyService})


SearchProxyServiceWrapper = type(
    'SearchProxyServiceWrapper',
    (ConcreteServiceWrapper,),
    {'concrete_service_class': SearchProxyService})


SubprocessSearchProxyServiceWrapper = type(
    'SubprocessSearchProxyServiceWrapper',
    (SubprocessConcreteServiceWrapper,),
    {'concrete_service_wrapper_class': SearchProxyServiceWrapper})


FeedbackClientWrapper = type(
    'FeedbackClientWrapper',
    (ConcreteServiceClientWrapper,),
    {'concrete_service_class': FeedbackService})


FeedbackServiceWrapper = type(
    'FeedbackServiceWrapper',
    (ConcreteServiceWrapper,),
    {'concrete_service_class': FeedbackService})


SubprocessFeedbackServiceWrapper = type(
    'SubprocessFeedbackServiceWrapper',
    (SubprocessConcreteServiceWrapper,),
    {'concrete_service_wrapper_class': FeedbackServiceWrapper})



#!/usr/bin/env python


"""
Deprecated: please use concrete-inspect.py instead.

Print human-readable information about a Communication to stdout

concrete_inspect.py is a command-line script for printing out information
about a Concrete Communication.
"""
from __future__ import print_function
from __future__ import unicode_literals

import argparse
import codecs
import sys

import concrete.version
import concrete.inspect
from concrete.util import CommunicationReader, FileType


sys.stderr.write(
    'concrete_inspect.py is deprecated and will be removed in the future.\n')
sys.stderr.write('Please use concrete-inspect.py instead.\n')


def print_header(header):
    print()
    print(header)
    print('-' * len(header))


def print_header_if(header, condition):
    if condition:
        print_header(header)


def main():
    # Make stdout output UTF-8, preventing "'ascii' codec can't encode" errors
    sys.stdout = codecs.getwriter('utf8')(sys.stdout)

    parser = argparse.ArgumentParser(
        description="Print information about a Concrete Communication to"
                    " stdout.  If communication_filename is specified, read"
                    " communication from file; otherwise, read from standard"
                    " input.",
    )
    parser.add_argument('--count', type=int,
                        help='Print at most this many communications.')
    parser.add_argument('--annotation-headers', action='store_true',
                        help='Print annotation type headers.')
    parser.add_argument("--char-offsets",
                        help="Print token text extracted from character offset"
                             "s (not the text stored in the tokenization) in '"
                             "ConLL-style' format",
                        action="store_true")
    parser.add_argument("--dependency",
                        help="Print HEAD and DEPREL tags for first dependency "
                             "parse in 'ConLL-style' format",
                        action="store_true")
    parser.add_argument("--dependency-tool",
                        type=str,
                        help='Filter --dependency output to specified '
                             'tool (requires --dependency)')
    parser.add_argument("--entities",
                        help="Print info about all Entities and their EntityMe"
                             "ntions",
                        action="store_true")
    parser.add_argument("--entities-tool",
                        type=str,
                        help='Filter --entities output to specified '
                             'tool (requires --entities)')
    parser.add_argument("--lemmas",
                        help="Print first set of lemma token tags in 'ConLL-st"
                             "yle' format",
                        action="store_true")
    parser.add_argument("--lemmas-tool",
                        type=str,
                        help='Filter --lemmas output to specified '
                             'tool (requires --lemmas)')
    parser.add_argument("--metadata",
                        help="Print metadata for tools used to annotate Commun"
                             "ication",
                        action="store_true")
    parser.add_argument("--metadata-tool",
                        type=str,
                        help='Filter --metadata output to specified '
                             'tool (requires --metadata)')
    parser.add_argument("--communication-taggings",
                        help="Print communication taggings",
                        action="store_true")
    parser.add_argument("--communication-taggings-tool",
                        type=str,
                        help='Filter --communication-taggings output to '
                             'specified tool (requires '
                             '--communication-taggings)')
    parser.add_argument("--mentions",
                        help="Print whitespace-separated tokens, with entity m"
                             "entions wrapped using <ENTITY ID=x> tags, where "
                             "'x' is the (zero-indexed) entity number",
                        action="store_true")
    parser.add_argument("--mentions-tool",
                        type=str,
                        help='Filter --mentions output to specified '
                             'tool (requires --mentions)')
    parser.add_argument("--ner",
                        help="Print first set of Named Entity Recognition toke"
                             "n tags in 'ConLL-style' format",
                        action="store_true")
    parser.add_argument("--ner-tool",
                        type=str,
                        help='Filter --ner output to specified '
                             'tool (requires --ner)')
    parser.add_argument("--pos",
                        help="Print first set of Part-Of-Speech token tags in "
                             "'ConLL-style' format",
                        action="store_true")
    parser.add_argument("--pos-tool",
                        type=str,
                        help='Filter --pos output to specified '
                             'tool (requires --pos)')
    parser.add_argument("--sections",
                        action='store_true',
                        help="Print text according to Section offsets"
                             "(textSpan values). These textSpans are assumed "
                             "to be valid.")
    parser.add_argument("--sections-tool",
                        type=str,
                        help='Filter --sections output to specified '
                             'tool (requires --sections)')
    parser.add_argument("--situation-mentions",
                        help="Print info about all SituationMentions",
                        action="store_true")
    parser.add_argument("--situation-mentions-tool",
                        type=str,
                        help='Filter --situation-mentions output to specified '
                             'tool (requires --situation-mentions)')
    parser.add_argument("--situations",
                        help="Print info about all Situations and their Situat"
                             "ionMentions",
                        action="store_true")
    parser.add_argument("--situations-tool",
                        type=str,
                        help='Filter --situations output to specified '
                             'tool (requires --situations)')
    parser.add_argument("--text",
                        help="Print .text field",
                        action="store_true")
    parser.add_argument("--text-tool",
                        type=str,
                        help='Filter --text output to specified '
                             'tool (requires --text)')
    parser.add_argument("--tokens",
                        help="Print whitespace-seperated tokens for *all* Toke"
                             "nizations in a Communication.  Each sentence tok"
                             "enization is printed on a separate line, and "
                             "empty lines indicate a section break",
                        action="store_true")
    parser.add_argument("--tokens-tool",
                        type=str,
                        help='Filter --tokens output to specified '
                             'tool (requires --tokens)')
    parser.add_argument("--treebank",
                        help="Print Penn-Treebank style parse trees for *all* "
                             "Constituent Parses in the Communication",
                        action="store_true")
    parser.add_argument("--treebank-tool",
                        type=str,
                        help='Filter --treebank output to specified '
                             'tool (requires --treebank)')
    parser.add_argument("--id",
                        help='Print communication id',
                        action='store_true')
    parser.add_argument("--id-tool",
                        type=str,
                        help='Filter --id output to specified '
                             'tool (requires --id)')
    parser.add_argument("--no-references",
                        help="Don't add references to communication (may preve"
                             "nt 'NoneType' errors)",
                        action="store_true")
    parser.add_argument('communication_filename',
                        nargs='?',
                        type=str,
                        help='Path to a Concrete Communication from which '
                        'to display information. If not specified, read '
                        'read from standard input')
    concrete.version.add_argparse_argument(parser)
    args = parser.parse_args()

    add_references = not args.no_references

    if args.communication_filename is not None:
        comms = CommunicationReader(args.communication_filename,
                                    add_references=add_references)
    else:
        comms = CommunicationReader('/dev/fd/0', add_references=add_references,
                                    filetype=FileType.STREAM)

    if not (args.char_offsets or args.dependency or args.lemmas or args.ner or
            args.pos or args.entities or args.mentions or args.metadata or
            args.sections or args.situation_mentions or args.situations or
            args.text or args.tokens or args.treebank or args.id or
            args.communication_taggings):
        parser.print_help()
        sys.exit(1)

    if ((args.dependency_tool and not args.dependency) or
            (args.lemmas_tool and not args.lemmas) or
            (args.ner_tool and not args.ner) or
            (args.pos_tool and not args.pos) or
            (args.entities_tool and not args.entities) or
            (args.mentions_tool and not args.mentions) or
            (args.metadata_tool and not args.metadata) or
            (args.sections_tool and not args.sections) or
            (args.situation_mentions_tool and not args.situation_mentions) or
            (args.situations_tool and not args.situations) or
            (args.text_tool and not args.text) or
            (args.tokens_tool and not args.tokens) or
            (args.treebank_tool and not args.treebank) or
            (args.id_tool and not args.id) or
            (args.communication_taggings_tool and
                not args.communication_taggings)):
        parser.print_help()
        sys.exit(1)

    comm_num = 0

    for (comm, _) in comms:
        if args.count is not None and comm_num == args.count:
            break

        if args.id:
            print_header_if('id', args.annotation_headers)
            concrete.inspect.print_id_for_communication(
                comm, tool=args.id_tool)
        if args.text:
            print_header_if('text', args.annotation_headers)
            concrete.inspect.print_text_for_communication(
                comm, tool=args.text_tool)
        if args.sections:
            print_header_if('sections', args.annotation_headers)
            concrete.inspect.print_sections(comm, tool=args.sections_tool)
        if args.tokens:
            print_header_if('tokens', args.annotation_headers)
            concrete.inspect.print_tokens_for_communication(
                comm, tool=args.tokens_tool)
        if args.treebank:
            print_header_if('treebank', args.annotation_headers)
            concrete.inspect.print_penn_treebank_for_communication(
                comm, tool=args.treebank_tool)
        if (args.char_offsets or args.dependency or args.lemmas or args.ner or
                args.pos):
            print_header_if('conll', args.annotation_headers)
            concrete.inspect.print_conll_style_tags_for_communication(
                comm, char_offsets=args.char_offsets,
                dependency=args.dependency,
                lemmas=args.lemmas, ner=args.ner, pos=args.pos,
                dependency_tool=args.dependency_tool,
                lemmas_tool=args.lemmas_tool,
                pos_tool=args.pos_tool,
                ner_tool=args.ner_tool)
        if args.entities:
            print_header_if('entities', args.annotation_headers)
            concrete.inspect.print_entities(comm, tool=args.entities_tool)
        if args.mentions:
            print_header_if('mentions', args.annotation_headers)
            concrete.inspect.print_tokens_with_entityMentions(
                comm, tool=args.mentions_tool)
        if args.situations:
            print_header_if('situations', args.annotation_headers)
            concrete.inspect.print_situations(comm, tool=args.situations_tool)
        if args.situation_mentions:
            print_header_if('situation mentions', args.annotation_headers)
            concrete.inspect.print_situation_mentions(
                comm, tool=args.situation_mentions_tool)
        if args.communication_taggings:
            print_header_if('communication taggings', args.annotation_headers)
            concrete.inspect.print_communication_taggings_for_communication(
                comm, tool=args.communication_taggings_tool)
        if args.metadata:
            print_header_if('metadata', args.annotation_headers)
            concrete.inspect.print_metadata(comm, tool=args.metadata_tool)

        comm_num += 1


if __name__ == "__main__":
    main()
